#include "CC_HashTable.h"
#include "stdlib.h"
#include "qpUtility.h"
#include "CC_Operators.h"

CCHashTable::CCHashTable(int cap)
{
	numBuckets=cap;
	numBits=Log2(cap)+1;
	ht=(LinkedList **)malloc(sizeof(LinkedList*)*numBuckets);
	int i=0;
	for(i=0;i<numBuckets;i++)
		ht[i]=NULL;
}

CCHashTable::~CCHashTable(void)
{
	int i=0;
	for(i=0;i<numBuckets;i++)
		if(ht[i]!=NULL)
			delete ht[i];
	free(ht);
}

//insert a key into the hash table. 
void CCHashTable::insertKey(Record* r)
{
	int bucketID=((r->value)&((1<<numBits)-1))%numBuckets;
	if(ht[bucketID]==NULL)
	{
		ht[bucketID]=(LinkedList*)malloc(sizeof(LinkedList));
		ht[bucketID]->init();
	}
		//ht[bucketID]=new LinkedList();
	ht[bucketID]->fill(r);
}

// probe a tuple against the hash table and store the results into Rout
int CCHashTable::probe(Record* r, Record** Rout)
{
	int result=0;
	int bucketID=((r->value)&((1<<numBits)-1))%numBuckets;
	if(ht[bucketID]!=NULL)
	{
		LinkedList *ll=ht[bucketID];
		//LinkedList *resultLL=(LinkedList*)malloc(sizeof(LinkedList));resultLL->init();
		LinkedList *resultLL=(LinkedList*)malloc(sizeof(LinkedList));
		resultLL->init();
		Bucket *sBucket=ll->header;
		int i=0;
		while(sBucket->next!=NULL)
		{
			for(i=0;i<NUM_ENTRY;i++)
			{
				if(sBucket->data[i].value==r->value)
				{
					resultLL->fill(sBucket->data+i);
					result++;
				}
			}
			sBucket=sBucket->next;
		}
		//the last bucket
		for(i=0;i<ll->curEntry;i++)
		if(sBucket->data[i].value==r->value)
		{
			resultLL->fill(sBucket->data+i);
			result++;
		}
		*Rout=(Record*)malloc(sizeof(Record)*result);
		resultLL->copyToArray(*Rout);
		delete resultLL;
	}
	return result;
}

int cc_constructHashTable(Record *Rin, int rLen, CCHashTable **ht)
{
	__DEBUG__("cc_constructHashTable");
	*ht=new CCHashTable(rLen/BUCKET_SIZE);
	int i=0;
	for(i=0; i<rLen; i++)
		(*ht)->insertKey(Rin+i);
	return rLen/BUCKET_SIZE;
}

int cc_hashSearch(Record *Rin, int rLen, CCHashTable *ht , int keyForSearch, Record** Rout)
{
	__DEBUG__("cc_hashSearch");
	Record *r=new Record();
	r->value=keyForSearch;
	int result=ht->probe(r,Rout);
	delete r;
	return result;
}

int cc_multi_hashSearch(Record *Rin, int rLen, CCHashTable *htable , int* searchKeys, int nKey, Record** Rout)
{
	__DEBUG__("cc_multi_hashSearch");
	int result=0;
	int bucketID;
	int k=0;
	int i=0;
	int keyForSearch;
	//LinkedList *resultLL=(LinkedList*)malloc(sizeof(LinkedList));resultLL->init();
	LinkedList *resultLL=(LinkedList*)malloc(sizeof(LinkedList));
	resultLL->init();
	int numBits=htable->numBits;
	int numBuckets=htable->numBuckets;
	for(k=0; k<nKey; k++)
	{
		keyForSearch=searchKeys[k];
		bucketID=((keyForSearch)&((1<<numBits)-1))%numBuckets;
		if(htable->ht[bucketID]!=NULL)
		{
			LinkedList *ll=htable->ht[bucketID];			
			Bucket *sBucket=ll->header;			
			while(sBucket->next!=NULL)
			{
				for(i=0;i<NUM_ENTRY;i++)
				{
					if(sBucket->data[i].value==keyForSearch)
					{
						resultLL->fill(sBucket->data+i);
						result++;
					}
				}
				sBucket=sBucket->next;
			}
			//the last bucket
			for(i=0;i<ll->curEntry;i++)
			if(sBucket->data[i].value==keyForSearch)
			{
				resultLL->fill(sBucket->data+i);
				result++;
			}
		}
	}
	*Rout=(Record*)malloc(sizeof(Record)*result);
	resultLL->copyToArray(*Rout);
	delete resultLL;
	return result;
}
