#include "CC_Operators.h"
#include "CO_Operators.h"
#include "stdlib.h"
#include "stdio.h"
#include "string.h"
#include "config.h"
#include "qpUtility.h"
#include "assert.h"
#include <iostream>
using namespace std;

#define SORT_BASE 16

//quicksort
void cc_quickSort(Record *Rin, int rLen, Record **Rout, int (*fcmp)(const void *, const void *))
{
	__DEBUG__("cc_quickSort");
	*Rout=(Record*)malloc(sizeof(Record)*rLen);
	memcpy(*Rout,Rin,sizeof(Record)*rLen);
	qsort(*Rout,rLen,sizeof(Record),fcmp);
}

/*void BinaryHashPartition(Record *R, int sR, int eR, int startBits)
{
	if(sR<eR && startBits>=0)
	{
		int mid=0;
		int left=sR;
		int right=eR;
		int curValue=0;
		Record tmp;
		while(left<right)
		{
			while(left<right)
			{
				 curValue=R[left].value;
				 if(((curValue>>startBits)& 1)==0)
					 left++;
				 else
					 break;
			}
			while(left<right)
			{
				 curValue=R[right].value;
				 if(((curValue>>startBits)& 1)==1)
					 right--;
				 else
					 break;
			}
			if(left<right)
			{
				SWAP(&R[left],&R[right],tmp);
				left++;
				right--;
			}
		}
		curValue=R[left].value;
		if(((curValue>>startBits)& 1)==0)
			mid=left;
		else
			mid=left-1;
		if(mid!=-1)
		BinaryHashPartition(R,sR,mid,startBits-1);
		BinaryHashPartition(R,mid+1,eR,startBits-1);
	}	
}*/

void HashPartition(Record *R, int sR, int eR, Record *dest,Record *swap, int startBits, int lenBits)
{
	if((eR-sR)>=SORT_BASE)
	{
		int i=0,j=0;
		int HASH_F=1<<lenBits;
	//	cout<<lenBits<<endl;
		int *boundary=new int[HASH_F];
		int *card=new int[HASH_F];
		for(i=0;i<HASH_F;i++)
			card[i]=0;
		int curValue=0;
		int pivot=1;
		for(i=0;i<lenBits;i++)
			pivot=pivot | (1<<i);
		int pID=0;
		int moveBits=startBits-lenBits;
		for(i=sR;i<eR;i++)
		{
			curValue=R[i].value;
			curValue=curValue>>moveBits;
			pID=curValue & pivot;
			card[pID]++;		
		}
		//distribution.
		int sum=0;
		for(i=0;i<HASH_F;i++)
		{
			boundary[i]=sum+sR;
			sum=sum+card[i];
		}
		//cout<<endl;
		for(i=sR;i<eR;i++)
		{
			curValue=R[i].value;
			curValue=curValue>>moveBits;
			pID=curValue & pivot;
			//dest[boundary[pID]].value=R[i].value;
			//dest[boundary[pID]].rid=R[i].rid;
			EVAL((dest+boundary[pID]),(R+i));
			boundary[pID]++;
		}
		/*cout<<"bits, "<<startBits<<"sR,"<<sR<<", eR"<<eR<<endl;
		for(i=sR;i<eR;i++)
			cout<<R[i].value<<endl;
		cout<<"after-->"<<endl;
		for(i=sR;i<eR;i++)
			cout<<dest[i].value<<endl;*/
		assert(boundary[HASH_F-1]==eR);
		startBits-=lenBits;
		if(startBits-lenBits>=0)
		{
			HashPartition(dest,sR,boundary[0],R,swap,startBits, lenBits);
			for(i=0;i<(HASH_F-1);i++)
				HashPartition(dest,boundary[i],boundary[i+1],R,swap,startBits,lenBits);
		}
		else
		{
			if(dest!=swap)
				memcpy(swap+sR,dest+sR,(eR-sR)*sizeof(Record));
			BinaryHashPartition(swap,sR,boundary[0]-1,startBits);
			for(i=0;i<(HASH_F-1);i++)
				BinaryHashPartition(swap,boundary[i],boundary[i+1]-1,startBits);
		}
		delete []card;
		delete []boundary;
	}
	else
	{
		if(R!=swap)
			memcpy(swap+sR,R+sR,(eR-sR)*sizeof(Record));
		BinaryHashPartition(swap,sR,eR-1,startBits);
		//qsort(swap+sR,(eR-sR),sizeof(Record),compareRecord);
	}
}


void cc_radixSort(Record *Rin, int rLen, Record **Rout)
{
	__DEBUG__("cc_radixSort");
	int totalBits=TOTAL_BITS;
	int lenBits=5;
	*Rout=(Record*)malloc(sizeof(Record)*rLen);
	HashPartition(Rin,0,rLen,*Rout,*Rout,totalBits, lenBits);
}

void cc_mergeSort(Record *Rin, int rLen, Record **Rout)
{
	__DEBUG__("cc_mergeSort");
	//int cacheChunk=L2_CAP/2/sizeof(Record);
	int cacheChunk=2;//L2_CAP/2/sizeof(Record);
	int numChunk=rLen/cacheChunk;
	if(rLen%cacheChunk!=0)
		numChunk=numChunk+1;
	int i=0, start, end;
	//sort
	*Rout=(Record*)malloc(sizeof(Record)*rLen);
	for(i=0; i<numChunk; i++)
	{
		start=i*cacheChunk;
		end=start+cacheChunk;
		if(end>rLen) end=rLen;
		qsort(Rin+start,(end-start),sizeof(Record),compareRecord);
	}
	//merge
	int mergeFanout=4;
	int numPass=0;//currently, we merge every four chunks.
	int tmpChunk=numChunk;
	while(tmpChunk>1)
	{
		tmpChunk=divRoundUp(tmpChunk, mergeFanout);
		numPass++;
	}
	tmpChunk=numChunk;
	int numStage=divRoundUp(tmpChunk, mergeFanout);
	int p=0;
	//Record **actHeader=(Record **)malloc(sizeof(Record *)*mergeFanout);
	int actHeader[4];
	int actCurPos[4];
	int actEnd[4];
	int values[4];
	Record *actSource, *actDest;
	int chunkSize=cacheChunk;
	int j;
	int min1, min2, min;
	for(p=0; p<numPass; p++)
	{
		if((p&1)==0)
		{
			actSource=Rin;
			actDest=*Rout;
		}
		else
		{
			actSource=*Rout;
			actDest=Rin;
		}
		for(i=0; i<numStage; i++)
		{
			for(j=0;j<4;j++)//fanout is 4.
			{
				actHeader[j]=(i*mergeFanout+j)*chunkSize;
				actCurPos[j]=0;
				actEnd[j]=chunkSize;
				if(actHeader[j]>=rLen)
					if(actHeader[j]>=(rLen+chunkSize))
						actEnd[j]=0;
					else
						actEnd[j]=rLen%chunkSize;
				//cout<<actHeader[j]<<actEnd[j]<<endl;
			}
			//merge them.
			int curPos=i*mergeFanout*chunkSize;
			end=curPos+mergeFanout*chunkSize;
			if(end>rLen) end=rLen;
			while(1)
			{
				for(j=0;j<4;j++)
				{
					if(actCurPos[j]!=actEnd[j])
					{
						values[j]=actSource[actCurPos[j]+actHeader[j]].value;
					}
					else
						values[j]=TEST_MAX;
				}
				min1=(values[0]>values[1])?1:0;
				min2=(values[2]>values[3])?3:2;
				min=(values[min1]>values[min2])?min2:min1;
				EVAL((actDest+curPos), (actSource+actCurPos[min]+actHeader[min]));
				//cout<<actDest[curPos].value<<endl;
				curPos++;
				actCurPos[min]++;
				if(curPos==end)//no more tuples.
					break;
			}
		}
		//update the statistics for next pass
		chunkSize=chunkSize*mergeFanout;
		tmpChunk=divRoundUp(tmpChunk, mergeFanout);
		numStage=divRoundUp(tmpChunk, mergeFanout);

	}

	//copy back
	if((p&1)==0)
	{
		memcpy(*Rout,Rin,sizeof(Record)*rLen);
	}
}

