// COPn.cpp: implementation of the COPn class.
//
//////////////////////////////////////////////////////////////////////

#include "qpUtility.h"
#include "COPn.h"
#include "CO_Operators.h"
#include "time.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

COPn::COPn()
{

}

COPn::~COPn()
{
	//release the resource. 
	int i=0;
	int baseID=0;
	int fanout=1;
	int j=0;
	delete []interBufferStatus;
	for(i=0;i<totalNumBuffers;i++)
		delete internalBuffers[i];
	delete internalBuffers;
	delete []baseIDList;
	baseIDList=new int[totalTreeLevel];
	for(i=0;i<(totalTreeLevel-1);i++)
	{
		baseID=(1<<(i+1))-2;
		baseIDList[i]=baseID;
		fanout=fanout<<1;
		for(j=0;j<fanout;j++)
		{
			delete internalBuffers[j+baseID];
		}
	}
	fanout=(1<<(totalTreeLevel));
	for(i=0;i<fanout;i++)
	{
		delete resultBuffers[i];
	}
	delete resultBuffers;
	

}

void COPn::postInitialize()
{
	int i=0;
	int baseID=0;
	int bufID=0;
	int fanout=1;
	int j=0;
	for(i=0;i<(totalTreeLevel-1);i++)
	{
		baseID=(1<<(i+1))-2;
		baseIDList[i]=baseID;
		fanout=fanout<<1;
		for(j=0;j<fanout;j++)
		{
			interBufferStatus[j+baseID]=0;	
		}
	}
	fanout=(1<<(totalTreeLevel));
	for(i=0;i<fanout;i++)
	{
		resultBuffers[i]->numBucket=0;
		resultBuffers[i]->curEntry=0;
	}
}
void COPn::Pn(Record *R, int rLen)
{
	int bufID;
	int i=0;
	int moveBits=totalHashLevel-1;
	Record *buffers[2];
	buffers[0]=internalBuffers[0];
	buffers[1]=internalBuffers[1];
	int size=bufSizeList[0];
	int nextI=0;
	for(i=0;i<rLen;i++)
	{
		bufID=(R[i].value>>moveBits) & 1;
		EVAL((buffers[bufID]+interBufferStatus[bufID]),(R+i));
		interBufferStatus[bufID]++;
		if((interBufferStatus[0])>=size)
		{
			flush(0,0);
			//interBufferStatus[0]=0;
		}
		else
		if((interBufferStatus[1])>=size)
		{
			flush(1,0);
			//interBufferStatus[1]=0;
		}
	}
}



void COPn::flush(int bufID, int level)
{
//#ifdef DEBUG
//	cout<<"flushing, "<<bufID<<", "<<level<<", "<<interBufferStatus[bufID]<<endl;
//#endif
	int size=interBufferStatus[bufID];
	Record *buf=internalBuffers[bufID];	
	//int moveBits=totalHashLevel-level-2;
	int moveBits=totalHashLevel-level-2;
	int tempBufID=0;
	int baseBufID=0;
	int i=0;	
	int nextID;
	int boundary=0;
	int tempNum=0;
	if((level+2)<totalTreeLevel)
	{
		baseBufID=(bufID<<1)+2;
		int nextLevel=level+1;
		int nextLevelSize=bufSizeList[nextLevel];
		if(interBufferStatus[baseBufID]+size<nextLevelSize && interBufferStatus[baseBufID+1]+size<nextLevelSize)
		{
			for(i=0;i<size;i++)
			{
				nextID=((buf[i].value>>moveBits) & 1)+baseBufID;
				EVAL((internalBuffers[nextID]+interBufferStatus[nextID]),(buf+i));
				interBufferStatus[nextID]++;
			}
			/*boundary=SimpleBinaryHashPartition(buf, 0, size-1, moveBits);
			if(boundary>=0)
			{
				tempNum=boundary+1;
				memcpy(internalBuffers[baseBufID]+interBufferStatus[baseBufID], buf, sizeof(Record)*tempNum);
				interBufferStatus[baseBufID]+=tempNum;
			}
			if((boundary+1)<size)
			{
				tempNum=size-boundary-1;
				baseBufID+=1;
				memcpy(internalBuffers[baseBufID]+interBufferStatus[baseBufID], buf+(boundary+1), sizeof(Record)*tempNum);
				interBufferStatus[baseBufID]+=tempNum;
			}*/
		}
		else
		{
			if((level+3)<totalTreeLevel)
			{
				int nextNextID=0;
				int nextMoveBits=moveBits+1;
				int nextNextLevel=nextLevel+1;
				int nextNextLevelSize=bufSizeList[nextNextLevel];
				for(i=0;i<size;i++)
				{
					nextID=((buf[i].value>>moveBits) & 1)+baseBufID;
					nextNextID=((buf[i].value>>nextMoveBits) & 1)+(nextID<<1)+2;
					EVAL((internalBuffers[nextNextID]+interBufferStatus[nextNextID]),(buf+i));
					interBufferStatus[nextNextID]++;
					if((interBufferStatus[nextNextID])==nextNextLevelSize)
					{
						flush(nextNextID,nextNextLevel);
						interBufferStatus[nextNextID]=0;
					}			
				}
			}
			else
			{
				for(i=0;i<size;i++)
				{
					nextID=((buf[i].value>>moveBits) & 1)+baseBufID;
					EVAL((internalBuffers[nextID]+interBufferStatus[nextID]),(buf+i));
					interBufferStatus[nextID]++;
					if((interBufferStatus[nextID])==nextLevelSize)
					{
						flush(nextID,nextLevel);
						interBufferStatus[nextID]=0;
					}			
				}
			}
		}
			
	}
	else
	{	
		for(i=0;i<size;i++)
		{
			//tempBufID=(buf[i].value>>HASH_BUCKET_BITS)& totalMask;
			tempBufID=(buf[i].value>>moveBits)& totalMask;
			resultBuffers[tempBufID]->fill(buf+i);
		}
	}
	interBufferStatus[bufID]=0;
	
}


void COPn::flushAll(int bufID, int level)
{
	if(interBufferStatus[bufID]!=0)
	flush(bufID,level);
	if((level+3)<totalTreeLevel)
	{
		int baseBufID=(bufID<<1)+2;
		int nextLevel=level+1;
		if(interBufferStatus[baseBufID]!=0)
		flush(baseBufID,nextLevel);
		int newbaseBufID=(baseBufID<<1)+2;
		nextLevel++;
		flushAll(newbaseBufID,nextLevel);
		flushAll(newbaseBufID+1,nextLevel);
		if(interBufferStatus[baseBufID+1]!=0)
		flush(baseBufID+1,level+1);
		flushAll(newbaseBufID+2,nextLevel);
		flushAll(newbaseBufID+3,nextLevel);
	}
	else
		if((level+3)==totalTreeLevel)
		{
			int baseBufID=(bufID<<1);
			flush(baseBufID+2,level+1);
			flush(baseBufID+3,level+1);
		}

}

void COPn::flushTree()
{
	flushAll(0,0);
	flushAll(1,0);
}



int COPn::COPSort(Record* R, int rLen, Record* output)
{
	int result=0;
	Pn(R,rLen);
	flushTree();
	int fanout=(1<<(totalTreeLevel));
	int i=0,j=0;
	int remain;
	int start=0;
	int end=0;
	int numBucket=0;
	for(j=0;j<fanout;j++)
	{
		LargeBucket *rBucket=resultBuffers[j]->header;
		int i=0;
		start=result;
		numBucket=0;
		while(numBucket<resultBuffers[j]->numBucket)
		{
			for(i=0;i<LARGE_NUM_ENTRY;i++)
			{
				EVAL((output+result),(rBucket->data+i));
				result++;
			}
			numBucket++;
			rBucket=rBucket->next;
		}
		//the last bucket
		remain=resultBuffers[j]->curEntry;
		for(i=0;i<remain;i++)
		{
			EVAL((output+result),(rBucket->data+i));
			result++;
		}
		end=result;
//#ifdef DEBUG
//		cout<<j<<", "<<start<<", "<<end<<":"<<end-start<<endl;
//#endif
		//sort the partition.
		if(start<end)
			qsort(output+start,(end-start),sizeof(Record),compareRecord);
	}
	return result;
}

int COPn::initialize(int rLen, int totalBits, int numLevel)
{
	totalHashLevel=totalBits;
	totalTreeLevel=numLevel;
	totalMask=(1<<(totalTreeLevel))-1;
	totalNumBuffers=(1<<(totalTreeLevel))-2;
//#ifdef DEBUG
//	cout<<"Hashing bits, "<<totalHashLevel<<", total level of buffers, "<<totalTreeLevel<<", mask, "<<totalMask<<endl;
//#endif
	interBufferStatus=new int[totalNumBuffers];
	internalBuffers=(Record **)malloc(sizeof(Record*)*totalNumBuffers);
	bufSizeList=new int[totalTreeLevel];
	//calculate the buffer size for the internal buffers of each level.
	//ComputeVEB(rLen,totalTreeLevel,bufSizeList);
	NewVEB(0,totalTreeLevel,totalTreeLevel,bufSizeList);
	int i=0;
	for(i=0;i<totalTreeLevel;i++)
		cout<<i<<", "<<bufSizeList[i]<<'\t';
	cout<<endl;
	
	int baseID=0;
	int bufID=0;
	int fanout=1;
	int j=0;
	//cout<<totalNumBuffers<<endl;
	baseIDList=new int[totalTreeLevel];
	for(i=0;i<(totalTreeLevel-1);i++)
	{
		baseID=(1<<(i+1))-2;
		baseIDList[i]=baseID;
		fanout=fanout<<1;
		for(j=0;j<fanout;j++)
		{
			internalBuffers[j+baseID]=new Record[bufSizeList[i]];
			interBufferStatus[j+baseID]=0;
		//	cout<<j+baseID<<", ";
		}
		//cout<<endl;
	}
	//fanout=(1<<(totalHashLevel-HASH_BUCKET_BITS));
	fanout=(1<<(totalTreeLevel));
	resultBuffers=(LargeVarBuffer **)malloc(sizeof(LargeVarBuffer*)*fanout);
	int numSeg=rLen/fanout/LARGE_NUM_ENTRY+1;
	for(i=0;i<fanout;i++)
	{
		resultBuffers[i]=new LargeVarBuffer(numSeg);
	}
	return 0;
}


int COPn::COPartition(Record* R, int rLen, int lowerBits, Record* output)
{
	int result=0;
	clock_t l_startTime = clock();
	Pn(R,rLen);
	clock_t l_endTime = clock();
	printf("Pn, %.3f ms, \n",(l_endTime-l_startTime)*1000/ (double)CLOCKS_PER_SEC);
	l_startTime = clock();
	flushTree();
	l_endTime = clock();
	printf("flushTree, %.3f ms, \n",(l_endTime-l_startTime)*1000/ (double)CLOCKS_PER_SEC);
	int fanout=(1<<(totalTreeLevel));
	int i=0,j=0;
	int remain;
	int start=0;
	int end=0;
	int numBucket=0;
	int numResultForThisBuffer=0;
	int left=0;
	int right=0;
	int localLowerBits=lowerBits-1;
	int remainderLowBits=lowerBits-2;	
	int jTimes=0, iTimes=0;
	int ijTimes=0;
	for(j=0;j<fanout;j++)
	{
		LargeBucket *rBucket=resultBuffers[j]->header;
		start=result;
		numResultForThisBuffer=(resultBuffers[j]->numBucket)*LARGE_NUM_ENTRY+resultBuffers[j]->curEntry;
		result=result+numResultForThisBuffer;
		end=result;
		numBucket=0;
		left=start;
		right=end-1;
		while(numBucket<resultBuffers[j]->numBucket)
		{
			for(i=0;i<LARGE_NUM_ENTRY;i++)
			{
				if(((rBucket->data[i].value>>localLowerBits)&1)==0)
				{
					EVAL((output+left),(rBucket->data+i));
					left++;
				}
				else
				{
					EVAL((output+right),(rBucket->data+i));
					right--;
				}
			}
			numBucket++;
			rBucket=rBucket->next;
		}
		//the last bucket
		
		remain=resultBuffers[j]->curEntry;
		for(i=0;i<remain;i++)
		{
			if(((rBucket->data[i].value>>localLowerBits)&1)==0)
			{
				EVAL((output+left),(rBucket->data+i));
				left++;
			}
			else
			{
				EVAL((output+right),(rBucket->data+i));
				right--;
			}
		}
		if(left!=(right+1))
		{
			__DEBUG__("error");
			exit(0);
		}
		//memcpy does not help.
		
		//further partition
		//make the bourdary to be [start, left), and [left, end)
		if(start<left)
		{
			//jTimes++;ijTimes++;
			BinaryHashPartition(output,start, (left-1), remainderLowBits);
			//__DEBUGInt2__(jTimes, ijTimes);
		}
		if(left<end)
		{
			//iTimes++;ijTimes++;
			BinaryHashPartition(output,left, (end-1), remainderLowBits);
			//__DEBUGInt2__(iTimes, ijTimes);
		}

	}
	return result;
}
