#include "CO_CSSTree.h"
#include "CO_Operators.h"
#include "stdlib.h"
#include "stdio.h"
#include "string.h"
#include "LinkedList.h"


int BFSTree::search(int key)
{
	int result=0;
	int i=0;
	int curIndex=0;
	//search
	for(i=0;i<level;i++)
	{
		if(key>tree[curIndex])
			curIndex=(curIndex<<1)+2;
		else
			curIndex=(curIndex<<1)+1;
	}
	//scan.
	curIndex=curIndex-numNode;
	for(i=curIndex;i<numRecord;i++)
		if(data[i].value==key)
			result++;
		else if(data[i].value > key)
			break;
	return result;
} 

void BFSTree::test(int nE)
{
	Record *data=new Record[nE];
	int i=0;
	for(i=0;i<nE;i++)
		data[i].value=i;
	BFSTree *gt=new BFSTree(data, nE);
	//gt->print();
	srand(0);
	int *myProbe=new int[nE];
	int numProbe=1024;
		
	for(int kk=0;kk<2;kk++)
	{
		for(i=0;i<numProbe;i++)
		myProbe[i]=RAND(nE);
		for(i=0;i<numProbe;i++)
			if(gt->search(myProbe[i])!=1)
			{
				cout<<"error bfs: "<<myProbe[i]<<endl;
				exit(0);
			}
		
	}
	
}

void BFSTree::print()
{
	int i=0;
	int start,end;
	int k=0;
	for(i=0;i<(level);i++)
	{
		start=(1<<i)-1;
		end=(1<<(i+1))-2;
		for(k=start;k<=end;k++)
		cout<<tree[k]<<", ";
		cout<<endl;
	}

}

int VEBTree::search(int key)
{
	int result=0;
	int i=0;
	int curIndex=0;
	int bfsFlag=0;
	//level=0;
	pos[0]=0;
	//cout<<key<<"->";
	if(key>vebtree[curIndex])
	{
		bfsFlag=(bfsFlag<<1)+1;//right
	}
	else
	{
		bfsFlag=(bfsFlag<<1)+0;//left
	}
	pos[1]=pos[vD[1]]+vT[1]+(bfsFlag & vT[1])*vB[1];
	curIndex=pos[1];
	//cout<<"1,"<<vebtree[pos[1]]<<"; ";
	
	for(i=2;i<=level;i++)
	{
		if(key>vebtree[curIndex])
		{
			bfsFlag=(bfsFlag<<1)+1;//right
		}
		else
		{
			bfsFlag=(bfsFlag<<1)+0;//left
		}
		pos[i]=pos[vD[i]]+vT[i]+(bfsFlag & vT[i])*vB[i];
		curIndex=pos[i];
		//cout<<i<<","<<vebtree[pos[i]]<<"; ";
	}
	//the last level
	if(key>vebtree[curIndex])
	{
		bfsFlag=(bfsFlag<<1)+1;//right
	}
	else
	{
		bfsFlag=(bfsFlag<<1)+0;//left
	}
	
	curIndex=bfsFlag;
	//assert(abs(curIndex-key)<5);
	//__DEBUG__(curIndex);
	/*for(i=curIndex;i<numRecord;i++)
		if(data[i].value==key)
			result++;
		else if(data[i].value > key)
			break;*/
//	cout<<endl;
	return curIndex;
} 

void VEBTree::test(int nE)
{
	Record *data=new Record[nE];
	int i=0;
	for(i=0;i<nE;i++)
		data[i].value=i;
	VEBTree *gt=new VEBTree(data, nE);
//	gt->print();
	int numProbe=1024;
	int *myProbe=new int[nE];
	for(int kk=0;kk<2;kk++)
	{
		for(i=0;i<numProbe;i++)
		myProbe[i]=RAND(nE);
		for(i=0;i<numProbe;i++)
			if(gt->search(myProbe[i])!=1)
			{
				cout<<"error veb"<<endl;
				exit(0);
			}
			/*else
				cout<<myProbe[i]<<endl;*/
		
	}
	
}

int co_constructCSSTree(Record *Rin, int rLen, VEBTree **tree)
{
	__DEBUG__("co_constructCSSTree");
	*tree=new VEBTree(Rin,rLen);
	return (*tree)->numNode;
}

int co_equiTreeSearch(Record *Rin, int rLen, VEBTree *tree, int keyForSearch, Record** Rout)
{
	__DEBUG__("co_equiTreeSearch");
	int result=0;
	int i=0;
	int curIndex=tree->search(keyForSearch);
	cout<<curIndex<<", ";
	LinkedList *ll=(LinkedList*)malloc(sizeof(LinkedList)); ll->init();
	for(i=curIndex-1;i>0;i--)
		if(Rin[i].value==keyForSearch)
		{
			ll->fill(Rin+i);
			result++;
		}
		else
			if(Rin[i].value<keyForSearch)
			break;
	for(i=curIndex;i<rLen;i++)
		if(Rin[i].value==keyForSearch)
		{
			ll->fill(Rin+i);
			result++;
		}
		else
			if(Rin[i].value>keyForSearch)
			break;
	(*Rout)=(Record*)malloc(sizeof(Record)*result);
	ll->copyToArray((*Rout));
	delete ll;
	return result;
}

int co_multi_equiTreeSearch(Record *Rin, int rLen, VEBTree *tree, int* searchKeys, int nKey, Record** Rout)
{
	__DEBUG__("co_multi_equiTreeSearch");
	int result=0;
	int i=0;
	LinkedList *ll=(LinkedList*)malloc(sizeof(LinkedList)); ll->init();
	int k=0;
	int curIndex=0;
	int keyForSearch;
	for(k=0; k<nKey; k++)
	{
		keyForSearch=searchKeys[k];
		curIndex=tree->search(keyForSearch);
		for(i=curIndex-1;i>0;i--)
			if(Rin[i].value==keyForSearch)
			{
				ll->fill(Rin+i);
				result++;
			}
			else
				if(Rin[i].value<keyForSearch)
				break;
		for(i=curIndex;i<rLen;i++)
			if(Rin[i].value==keyForSearch)
			{
				ll->fill(Rin+i);
				result++;
			}
			else
				if(Rin[i].value>keyForSearch)
				break;
	}
	(*Rout)=(Record*)malloc(sizeof(Record)*result);
	ll->copyToArray((*Rout));
	delete ll;
	return result;
}



