#ifndef CSSTREE_H
#define CSSTREE_H

#include "db.h"


#include "assert.h"
#include "qpUtility.h"
#include "math.h"
#include "stdlib.h"
#include <iostream>
using namespace std;


#define TREE_FANOUT 33
//we use implicit pointer to perform the addressing.
class CO_GenericTree
{
public:
	int numRecord;
	int oldNumRecord;
	Record *data;
	//we use the BFS layout as the default layout.
	int numNode;
	int level;
	int gResult;
	CO_GenericTree(){}
	//we assume that numR=2^i. Otherwise, we pad the array with -1 from the beginning.
	//we also assume that the record are sorted by the key.
	CO_GenericTree(Record *d, int numR)
	{
		data=d;
		oldNumRecord=numR;
		if(!is2n(numR))
		{
			int mi=Log2(numR);
            numRecord=1<<(mi+1);	
		}
		else
			numRecord=numR;	

	}
	~CO_GenericTree()
	{
	}
	virtual int search(int key)=0;

};

class BFSTree:public CO_GenericTree
{
public:
	int *tree;
	void print();
	static void test(int nE);
	BFSTree(Record *d, int numR):CO_GenericTree(d,numR)
	{
		//build the tree.
		numNode=numRecord-1;
		level=Log2(numNode);
		tree=new int[numNode];
		int start;
		int end;
		int j=0;
		int k=0;
		int m=0;
		int curIndex=0;
		int i=0;
/*#ifdef DEBUG
		cout<<" #node, "<<numNode<<", #level, "<<level<<endl;
#endif	*/	
		for(i=0;i<=(level);i++)
		{
			start=(1<<i)-1;
			end=(1<<(i+1))-2;
			for(k=start;k<=end;k++)
			{
				curIndex=(k<<1)+1;//to left.
				for(m=i+1;m<=level;m++)
					curIndex=(curIndex<<1)+2;//to right.
				curIndex=curIndex-numNode;
				if(curIndex < oldNumRecord)
					tree[k]=data[curIndex].value;
				else
					tree[k]=TEST_MAX;
				//cout<<curIndex<<",";
			}
			//cout<<endl;
		}
//#ifdef DEBUG
//		cout<<"constructed..."<<endl;
//#endif	
	}
	virtual int search(int key);
};

class VEBTree:public BFSTree
{
public:
	int vebPos;//only for constructing the veb. marking the current position.
	int *vebtree;
	int* vB;//size of bottom tree
	int* vT;//size of top tree
	int* vD;//the depth of root of the coresp. tree.
	int* pos;
	VEBTree(Record *d, int numR):BFSTree(d,numR)
	{		
		//layout the tree according to VEB recursion.
		vB=new int[level];
		vT=new int[level];
		vD=new int[level];
		pos=new int[level];
		vebtree=new int[numNode];
		vebPos=0;
		placeVEB(0,0,level);	
	}
	~VEBTree()
	{
	}
	//[fromLevel, toLevel]. If fromLevel=toLevel, we can stop the recursion and layout
	//the tree directly.
	void placeVEB(int rootID, int fromLevel, int toLevel)
	{
		assert(fromLevel<=toLevel);
		//cout<<"ID, "<<vebPos<<", "<<rootID<<", "<<fromLevel<<", "<<toLevel<<endl;
		if(fromLevel==toLevel)
		{	
			vebtree[vebPos]=tree[rootID];
			vebPos++;
			/*vD[toLevel]=fromLevel;
			vT[toLevel]=1;
			vB[toLevel]=0;*/
		}
		else if((fromLevel+1)==toLevel)
		{
			vebtree[vebPos]=tree[rootID];
			vebPos++;
			vebtree[vebPos]=tree[(rootID<<1)+1];
			vebPos++;
			vebtree[vebPos]=tree[(rootID<<1)+2];
			vebPos++;
			vD[toLevel]=fromLevel;
			vT[toLevel]=1;
			vB[toLevel]=1;
		}
		else
		{
			int midH=divRoundUp((fromLevel+toLevel),2);
			vD[midH+1]=fromLevel;
			vT[midH+1]=(1<<(midH-fromLevel+1))-1;
			vB[midH+1]=(1<<(toLevel-midH))-1;
			placeVEB(rootID,fromLevel,midH);
			int i=0;
			int nextLevel=midH+1;
			int start=rootID;
			int end=rootID;
			for(i=fromLevel;i<=midH;i++)
			{
				start=(start<<1)+1;
				end=(end<<1)+2;
			}
			
			for(i=start;i<=end;i++)
				placeVEB(i,midH+1,toLevel);			
		}
	}
	int search(int key);
	
	static void test(int nE);

	void print()
	{
		int i=0;
		for(i=0;i<numNode;i++)
			cout<<tree[i]<<", ";
		cout<<endl;
		for(i=0;i<numNode;i++)
			cout<<vebtree[i]<<", ";
		cout<<endl;

		//cout<<"\tB"<<"\tT"<<"\tD"<<endl;
		//for(i=0;i<level;i++)
		//	cout<<i<<", "<<vB[i]<<","<<vT[i]<<","<<vD[i]<<endl;
			
	}


};


#endif

