#include "CO_Operators.h"
#include "stdlib.h"
#include "stdio.h"
#include "string.h"
#include "config.h"
#include "qpUtility.h"
#include "assert.h"
#include "LinkedList.h"
#include "CO_CSSTree.h"
#include "CO_HashTable.h"
#include "evaluateFuction.h"
#include "COPn.h"
#include "time.h"
#include "Operators.h"
#include "MyPCL.h"
#include <iostream>
using namespace std;


extern double s_excludeTime;
/*
We return the result <R.rid, S.rid> for further construction the result.
*/

IN_INT_OUT_INT PEVAL;

int CONLJ(Record *R, Record *S, int rStart, int rEnd, int sStart, int sEnd, PREDICATE_NODE * pRoot, LinkedList *ll, int *stack)
{
	int result=0;
	if(rStart+CUTOFF>=rEnd)
	{
		int data[2];
		Record r;
		for(int i=sStart;i<sEnd;i++)
		{
			data[1]=S[i].value;	
			for(int j=rStart;j<rEnd;j++)
			{
				data[0]=R[j].value;	
#ifdef NON_LINUX_PLAT
				if(evaluate(data,pRoot, stack, MAX_PREDICATE_NODE))
#else
				if(PEVAL(data))
#endif
				{	
					r.rid=R[j].rid;
					r.value=S[i].rid;
					ll->fill(&r);
					result++;				
				}
			}
		}
	}
	else
		if(sStart+CUTOFF>=sEnd)
		{
			int data[2];
			Record r;
			for(int j=rStart;j<rEnd;j++)			
			{
				data[0]=R[j].value;	
				for(int i=sStart;i<sEnd;i++)
				{
					data[1]=S[i].value;	
#ifdef NON_LINUX_PLAT
				if(evaluate(data,pRoot, stack, MAX_PREDICATE_NODE))
#else
				if(PEVAL(data))
#endif
					{	
						r.rid=R[j].rid;
						r.value=S[i].rid;
						ll->fill(&r);
						result++;				
					}
				}
			}
		}
		else
		{
			int midS=(sStart+sEnd)/2;
			int midR=(rStart+rEnd)/2;
			//interesting order
			result+=CONLJ(R,S,rStart,midR,sStart,midS,pRoot,ll,stack);
			result+=CONLJ(R,S,midR,rEnd,sStart,midS,pRoot,ll,stack);
			result+=CONLJ(R,S,midR,rEnd,midS,sEnd,pRoot,ll,stack);
			result+=CONLJ(R,S,rStart,midR,midS,sEnd,pRoot,ll,stack);
		}
	return result;
}

int co_ninlj(Record *R, int rLen, Record *S, int sLen, PREDICATE_NODE * pRoot, Record** Rout)
{
	__DEBUG__("co_ninlj");
	int numResult = 0;
	int i=0;
	int j=0;
	int k=0;
	int m=0;
	int startIndex=0;
	int endIndex=0;
	__DEBUG2__("initialPredicate", pRoot->CC_str);
	initialPredicate(pRoot->CC_str);
	void **dllHandle=(void**)malloc(sizeof(void*));
	PEVAL=openDLL(dllHandle);
	LinkedList *ll=(LinkedList*)malloc(sizeof(LinkedList)); ll->init();
	int stack[128];
	numResult=CONLJ(R,S,0,rLen,0,sLen,pRoot,ll,stack);
	*Rout=(Record*)malloc(sizeof(Record)*numResult);
	ll->copyToArray(*Rout);
	delete ll;
	closeDLL(dllHandle);
	return numResult;
}


int multi_CONLJ(Record **input,int num_col, int rNumColumn, int rStart, int rEnd, int sStart, int sEnd, PREDICATE_NODE * pRoot, LinkedList *ll, int *stack)
{
	int result=0;
	if(rStart+CUTOFF>=rEnd)
	{
		int *data=(int*)malloc(sizeof(int)*num_col);
		Record r;
		int col=0;
		int lastColumn=num_col-1;
		for(int i=sStart;i<sEnd;i++)
		{
			for(col=rNumColumn;col<num_col;col++)
					data[col]=input[col][i].value;			
			for(int j=rStart;j<rEnd;j++)
			{
				for(col=0;col<rNumColumn;col++)
					data[col]=input[col][j].value;	
#ifdef NON_LINUX_PLAT
				if(evaluate(data,pRoot, stack,MAX_PREDICATE_NODE))
#else
				if(PEVAL(data))
#endif
				{	
					r.rid=input[0][j].rid;
					r.value=input[lastColumn][j].rid;
					ll->fill(&r);
					result++;				
				}
			}
		}
		free(data);
	}
	else
		if(sStart+CUTOFF>=sEnd)
		{
			int *data=(int*)malloc(sizeof(int)*num_col);
			Record r;
			int col=0;
			int lastColumn=num_col-1;
			for(int j=rStart;j<rEnd;j++)
			{
				for(col=0;col<rNumColumn;col++)
						data[col]=input[col][j].value;			
				for(int i=sStart;i<sEnd;i++)
				{
					for(col=rNumColumn;col<num_col;col++)
						data[col]=input[col][i].value;						
					//if(R[j].value==S[i].value)
#ifdef NON_LINUX_PLAT
				if(evaluate(data,pRoot, stack,MAX_PREDICATE_NODE))
#else
				if(PEVAL(data))
#endif
				{	
						r.rid=input[0][j].rid;
						r.value=input[lastColumn][j].rid;
						ll->fill(&r);
						result++;				
					}
				}
			}
			free(data);
		}
		else
		{
			int midS=(sStart+sEnd)/2;
			int midR=(rStart+rEnd)/2;
			//interesting order
			result+=multi_CONLJ(input,num_col,rNumColumn,rStart,midR,sStart,midS,pRoot,ll,stack);
			result+=multi_CONLJ(input,num_col,rNumColumn,midR,rEnd,sStart,midS,pRoot,ll,stack);
			result+=multi_CONLJ(input,num_col,rNumColumn,midR,rEnd,midS,sEnd,pRoot,ll,stack);
			result+=multi_CONLJ(input,num_col,rNumColumn,rStart,midR,midS,sEnd,pRoot,ll,stack);
		}
	return result;
}

int co_multiNLJ(Record **input,int num_col, int rNumColumn, int rLen,int sLen, PREDICATE_NODE * pRoot, Record ** Rout)
{
	__DEBUG__("co_multiNLJ");
	int numResult = 0;
	int i=0;
	int j=0;
	int k=0;
	int m=0;
	int startIndex=0;
	int endIndex=0;
	__DEBUG2__("initialPredicate", pRoot->CC_str);
	initialPredicate(pRoot->CC_str);
	void **dllHandle=(void**)malloc(sizeof(void*));
	PEVAL=openDLL(dllHandle);
	LinkedList *ll=(LinkedList*)malloc(sizeof(LinkedList)); ll->init();
	int stack[128];
	numResult=multi_CONLJ(input,num_col,rNumColumn,0,rLen,0,sLen,pRoot,ll,stack);
	*Rout=(Record*)malloc(sizeof(Record)*numResult);
	ll->copyToArray(*Rout);
	delete ll;
	closeDLL(dllHandle);
	return numResult;

}
//the indexed relation is R!!
int co_inlj(Record *R, int rLen, VEBTree *tree, Record *S, int sLen, PREDICATE_NODE * pRoot, Record** Rout)
{
	__DEBUG__("co_inlj");
	int result=0;
	int i=0;
	LinkedList *ll=(LinkedList*)malloc(sizeof(LinkedList)); ll->init();
	int k=0;
	int curIndex=0;
	int keyForSearch;
	Record r;
	int data[2];
	for(k=0; k<sLen; k++)
	{
		keyForSearch=S[k].value;
		curIndex=tree->search(keyForSearch);
		//assert(abs(k-curIndex)<5);
		//__DEBUG__(curIndex);
		data[1]=keyForSearch;
		for(i=curIndex-1;i>0;i--)
		{
			data[0]=R[i].value;
			if(R[i].value==keyForSearch)
			//if(evaluate(data,pRoot))
			{
				r.rid=R[i].rid;
				r.value=S[k].rid;
				ll->fill(&r);
				result++;
			}
			else
				if(R[i].value<keyForSearch)
				break;
		}
		for(i=curIndex;i<rLen;i++)
		{	
			data[0]=R[i].value;
			if(R[i].value==keyForSearch)
			//if(evaluate(data,pRoot))
			{
				r.rid=R[i].rid;
				r.value=S[k].rid;
				ll->fill(&r);
				result++;
			}
			else
				if(R[i].value>keyForSearch)
				break;
		}
	}
	(*Rout)=(Record*)malloc(sizeof(Record)*result);
	ll->copyToArray((*Rout));
	delete ll;
	return result;
}
/*
mergeJoin: merge the two sorted relations. 
*/
/*int mergeJoin(Record *R, int rLen, Record *S, int sLen, Record** Rout)
{
	int result=0;
	LinkedList *ll=(LinkedList*)malloc(sizeof(LinkedList)); ll->init();
	int pR=0;
	int pS=0;
	int pS2=0;
	Record r;
	while(pR<rLen && pS<sLen)
	{
		if(R[pR].value==S[pS].value)
		{
			pS2=pS;
			r.rid=R[pR].rid;
			r.value=S[pS].rid;
			ll->fill(&r);
			result++;
			pS2++;
			while (pS2<sLen) {
				if(R[pR].value==S[pS2].value)
				{
					r.rid=R[pR].rid;
					r.value=S[pS2].rid;
					ll->fill(&r);
					result++;
					pS2++;				
				}
				else
				break;
			}
			pR++;
		}
		else if(R[pR].value<S[pS].value)
			pR++;
		else
			pS++;
	}
	(*Rout)=(Record*)malloc(sizeof(Record)*result);
	ll->copyToArray((*Rout));
	delete ll;
	return result;
}*/
int co_smj(Record *R, int rLen, Record *S, int sLen, PREDICATE_NODE * pRoot, Record** Rout)
{
	__DEBUG__("co_smj");
	Record **sortedR=(Record**)malloc(sizeof(Record*));
	Record **sortedS=(Record**)malloc(sizeof(Record*));
	co_partitionSort(R,rLen,sortedR);
	co_partitionSort(S,sLen,sortedS);
	int result=co_mergeJoin(*sortedR,rLen,*sortedS,sLen,Rout);
	delete (*sortedR);
	delete (*sortedS);
	free(sortedR);
	free(sortedS);
	return result;
}
//R is the smaller relation.
int co_simpleHJ(Record *R, int rLen, Record *S, int sLen, Record** Rout)
{
	__DEBUG__("co_simpleHJ");
	int result=0;
	int bucketID;
	int k=0;
	int i=0;
	int keyForSearch;
	LinkedList *resultLL=(LinkedList*)malloc(sizeof(LinkedList));resultLL->init();
	Record r;
	COHashTable **htable=(COHashTable**)malloc(sizeof(COHashTable*));
	co_constructHashTable(R, rLen, htable);
	int numBits=(*htable)->numBits;
	int numBuckets=(*htable)->numBuckets;
	for(k=0; k<sLen; k++)
	{
		keyForSearch=S[k].value;
		bucketID=((keyForSearch)&((1<<numBits)-1))%numBuckets;
		if((*htable)->ht[bucketID]!=NULL)
		{
			LinkedList *ll=(*htable)->ht[bucketID];			
			Bucket *sBucket=ll->header;			
			while(sBucket->next!=NULL)
			{
				for(i=0;i<NUM_ENTRY;i++)
				{
					if(sBucket->data[i].value==keyForSearch)
					{
						r.rid=sBucket->data[i].rid;
						r.value=S[k].rid;
						resultLL->fill(&r);
						result++;
					}
				}
				sBucket=sBucket->next;
			}
			//the last bucket
			for(i=0;i<ll->curEntry;i++)
			if(sBucket->data[i].value==keyForSearch)
			{
				r.rid=sBucket->data[i].rid;
				r.value=S[k].rid;
				resultLL->fill(&r);
				result++;
			}
		}
	}
	*Rout=(Record*)malloc(sizeof(Record)*result);
	resultLL->copyToArray(*Rout);
	delete resultLL;
	return result;
}
/*
if both the relations are larger, we use radix join.
*/
int copj(Record *RR, int rLen, Record *SS, int sLen, PREDICATE_NODE * pRoot, Record** Rout)
{
	__DEBUG__("copj");
	s_excludeTime=0;
	MyPCL *pcl=new MyPCL();
	pcl->OpenPCL();
	clock_t l_startTime2 = clock();	
	int result=0;
	int maxBits=Log2(rLen);
	maxBits=((maxBits&1)==0)?maxBits: (maxBits+1);
	maxBits=maxBits;
	int pivot=(1<<(maxBits))-1;
	int numLevel=maxBits/2;
	//we leave the <DIRECT_LEVEL> to directly NLJ.
	int lowerBits=maxBits-numLevel;
	Record* swapR=new Record[rLen];
	Record* swapS=new Record[sLen];
	COPn *cop=new COPn();
	clock_t l_startTime = clock();
	cop->initialize(rLen,maxBits,numLevel);
	clock_t l_endTime = clock();
	printf("initialize time, %.3f ms, \n",(l_endTime-l_startTime)*1000/ (double)CLOCKS_PER_SEC);
	s_excludeTime=pcl->ClosePCL();
	delete pcl;
	
	l_startTime = clock();
	int num1=cop->COPartition(RR,rLen,lowerBits,swapR);
	l_endTime = clock();
	printf("first partition time, %.3f ms, \n",(l_endTime-l_startTime)*1000/ (double)CLOCKS_PER_SEC);
	l_startTime = clock();
	//cop->COPSort(RR, rLen,swapR);
	//checkSorted(swapR,rLen);
	cop->postInitialize();
	int num2=cop->COPartition(SS,sLen,lowerBits,swapS);
	l_endTime = clock();
	printf("partition time, %.3f ms, \n",(l_endTime-l_startTime)*1000/ (double)CLOCKS_PER_SEC);
	//cop->COPSort(SS, sLen,swapS);
	//checkSorted(swapS,sLen);
		
	Record *R=swapR;
	Record *S=swapS;
	
	/*clock_t l_startTime = clock();
	BinaryHashPartition(RR, 0,rLen-1,maxBits);
	BinaryHashPartition(SS, 0,sLen-1,maxBits);
	Record *R=RR;
	Record *S=SS;
	clock_t l_endTime = clock();
	printf("initialize time, %.3f ms, \n",(l_endTime-l_startTime)*1000/ (double)CLOCKS_PER_SEC);*/

	l_startTime = clock();	
	result=maskMergeJoin(R,rLen,S,sLen,pivot,Rout);
	l_endTime = clock();
	printf("merge time, %.3f ms, \n",(l_endTime-l_startTime)*1000/ (double)CLOCKS_PER_SEC);
	
	printf("total time, %.3f ms, \n",(l_endTime-l_startTime2)*1000/ (double)CLOCKS_PER_SEC);
	/*delete []swapR;
	delete []swapS;*/
	
	return result;

}

int co_hj(Record *R, int rLen, Record *S, int sLen, PREDICATE_NODE * pRoot, Record** Rout)
{
	__DEBUG__("co_hj");
	int result=0;
	if(rLen<=256)
	{
		result=co_simpleHJ(R,rLen,S,sLen,Rout);
	}
	else
	{
		result=copj(R,rLen,S,sLen,pRoot, Rout);
	}
	return result;

}

/*
mergeJoin: merge the two sorted relations. 
*/
int co_mergeJoin(Record *R, int rLen, Record *S, int sLen, Record** Rout)
{
	__DEBUG__("mergeJoin");
	int result=0;
	LinkedList *ll=(LinkedList*)malloc(sizeof(LinkedList)); ll->init();
	int pR=0;
	int pS=0;
	int pS2=0;
	Record r;
	while(pR<rLen && pS<sLen)
	{
		if(R[pR].value==S[pS].value)
		{
			pS2=pS;
			r.rid=R[pR].rid;
			r.value=S[pS].rid;
			ll->fill(&r);
			result++;
			pS2++;
			while (pS2<sLen) {
				if(R[pR].value==S[pS2].value)
				{
					r.rid=R[pR].rid;
					r.value=S[pS2].rid;
					ll->fill(&r);
					result++;
					pS2++;				
				}
				else
				break;
			}
			pR++;
		}
		else if(R[pR].value<S[pS].value)
			pR++;
		else
			pS++;
	}
	(*Rout)=(Record*)malloc(sizeof(Record)*result);
	ll->copyToArray((*Rout));
	delete ll;
	return result;
}
