#include "CO_Operators.h"
#include "CC_Operators.h"
#include "stdlib.h"
#include "stdio.h"
#include "string.h"
#include "config.h"
#include "qpUtility.h"
#include "assert.h"
#include "COPn.h"
#include <iostream>
using namespace std;

#define SORT_BASE 16

//quicksort
void co_quickSort(Record *Rin, int rLen, Record **Rout, int (*fcmp)(const void *, const void *))
{
	__DEBUG__("co_quickSort");
	*Rout=(Record*)malloc(sizeof(Record)*rLen);
	memcpy(*Rout,Rin,sizeof(Record)*rLen);
	qsort(*Rout,rLen,sizeof(Record),fcmp);
//	BinaryRangePartition(*Rout,0,rLen-1,fcmp);

}

static int compareValueForPartitionMask;
int compareValueForPartition (const void * a, const void * b)
{
	return ( (((Record*)a)->value &compareValueForPartitionMask) - (((Record*)b)->value &compareValueForPartitionMask) );
}


void BinaryHashPartition(Record *R, int sR, int eR, int startBits)
{
	int mid=0;
	int left=sR;
	int right=eR;
	int curValue=0;
	Record tmp;
	while(left<right)
	{
		while(left<right)
		{
			 curValue=R[left].value;
			 if(((curValue>>startBits)& 1)==0)
				 left++;
			 else
				 break;
		}
		while(left<right)
		{
			 curValue=R[right].value;
			 if(((curValue>>startBits)& 1)==1)
				 right--;
			 else
				 break;
		}
		if(left<right)
		{
			SWAP(&R[left],&R[right],tmp);
			left++;
			right--;
		}
	}
	curValue=R[left].value;
	if(((curValue>>startBits)& 1)==0)
		mid=left;
	else
		mid=left-1;
	if(startBits>0)
	{
		//if(mid!=-1)
		if(mid>sR)
		BinaryHashPartition(R,sR,mid,startBits-1);
		if((mid+1)<eR)
		BinaryHashPartition(R,mid+1,eR,startBits-1);
	}
}

//[sR,mid], (mid,eR)
int SimpleBinaryHashPartition(Record *R, int sR, int eR, int startBits)
{
	int mid=0;
	int left=sR;
	int right=eR;
	int curValue=0;
	Record tmp;
	while(left<right)
	{
		while(left<right)
		{
			 curValue=R[left].value;
			 if(((curValue>>startBits)& 1)==0)
				 left++;
			 else
				 break;
		}
		while(left<right)
		{
			 curValue=R[right].value;
			 if(((curValue>>startBits)& 1)==1)
				 right--;
			 else
				 break;
		}
		if(left<right)
		{
			SWAP(&R[left],&R[right],tmp);
			left++;
			right--;
		}
	}
	curValue=R[left].value;
	if(((curValue>>startBits)& 1)==0)
		mid=left;
	else
		mid=left-1;
	return mid;
}




void co_radixSort(Record *Rin, int rLen, Record **Rout)
{
	__DEBUG__("co_radixSort");
	int totalBits=TOTAL_BITS;
	int lenBits=5;
	*Rout=(Record*)malloc(sizeof(Record)*rLen);
	BinaryHashPartition(Rin,0,rLen-1,totalBits);
	memcpy(*Rout,Rin,rLen*sizeof(Record));
}

void co_partitionSort(Record *R,int rLen, Record **Rout)
{
	__DEBUG__("co_partitionSort");
	COPn *cop=new COPn();
	int totalBits=TOTAL_BITS;
	int numLevel=Log2(rLen)/2;
	cop->initialize(rLen,totalBits,numLevel);
	*Rout=(Record*)malloc(sizeof(Record)*rLen);
	cop->COPSort(R,rLen,*Rout);
}

