#include "DBArray.h"
#include "stdlib.h"
#include "assert.h"
#include <iostream>
#include <fstream>
using namespace std;

DBArray::DBArray(void)
: numAttr(0)
, data(NULL)
, numTuple(0)
{
}

DBArray::~DBArray(void)
{
	if(data!=NULL)
		delete data;
}

DBArray::DBArray(int nA, int rLen)
{
	this->numAttr=nA;
	this->numTuple=rLen;
	this->data=new int[(nA+1)*rLen];
	assert(data!=NULL);
}

// this function imports the data from file. the first line of the file should be <numAttributes><space><numTuples>
void DBArray::importDB(char* dbFile)
{
	ifstream inf(dbFile);
	assert(inf);
	int nA=-1;
	int rLen=-1;
	inf>>nA;
	inf>>rLen;
	assert(nA>0 && rLen>0);
	this->numAttr=nA;
	this->numTuple=rLen;
	this->data=new int[(nA+1)*rLen];
	int i=0;
	while(!inf.eof())
	{
		inf>>(this->data[i]);
		i++;
	}
	inf.close();
}

// perform a simple check on the array
bool DBArray::check(void)
{
	bool bResult=true;
	int numEntry=(this->numAttr+1)*this->numTuple;
	int i=0;
	for(i=0;i<numEntry;i++)
		if(this->data[i]<0)
		{
			bResult=false;
			break;
		}
	return bResult;
}

// export to a Record array; return the number of tuples.
//the attribute index is based on one!!!
int DBArray::extractToRecord(Record ** R, int attrIndex)
{
	int i=0, j=0;
	int delta=this->numAttr+1;
	*R=new Record[this->numTuple];
	assert(R!=NULL);
	for(i=0;i<this->numTuple;i++)
	{
		(*R)[i].rid=this->data[j];
		(*R)[i].value=this->data[j+attrIndex];
		j=j+delta;
	}
	return this->numTuple;
}

void DBArray::transform(DBArray *in, Record *R, int rLen, DBArray *out)
{	
	out->init(in->numAttr,rLen);
	int i=0;
	int copyUnitSize=(in->numAttr+1)*sizeof(int);
	int rid=0;
	int delta=in->numAttr+1;
	for(i=0;i<rLen;i++)
	{
		rid=R[i].rid;
		memcpy(out->data+i*delta, in->data+rid*delta, copyUnitSize);
	}
}

void DBArray::init(int nA, int rLen)
{
	assert(data==NULL);
	this->numAttr=nA;
	this->numTuple=rLen;
	this->data=new int[(nA+1)*rLen];
	assert(data!=NULL);
}




