#include "DemoTest.h"

#include <iostream>
#include "Database.h"
#include "UInterface.h"
#include "QueryPlanTree.h"
#include "evaluateFuction.h"
#include "QueryPlanNode.h"
#include "MyPCL.h"

//CC
#include "CC_Operators.h"
#include "CC_CSSTree.h"
//CO
#include "CC_Operators.h"
#include "CC_CSSTree.h"

#include "batchQuery.h"

#include "testPrimitive.h"
#include "DBArray.h"
#include "RowExec.h"
#include "UnitTest.h"


using namespace std;

extern Database *easedb;
extern char *str;
extern MyPCL *pcl;


void evaluateQuery(int index, char *query)
{
	MyPCL *pcl=(MyPCL*)malloc(sizeof(MyPCL));
	pcl->OpenPCL();
	QueryPlanTree tree(CO_EXEC);
	tree.buildTree(query);
	tree.execute();
	pcl->ClosePCL();

	int len=tree.planStatus->numResultColumn*tree.planStatus->numResultRow;
	__DEBUGInt__(len);
	int *result=tree.planStatus->finalResult;
	char qName[1024];
	sprintf(qName,"Q%d.txt",index);
	FILE *dbFile = fopen(qName, "w");
	int row=tree.planStatus->numResultRow;
	int column=tree.planStatus->numResultColumn;
	int i=0,j=0;
	for(i=0;i<16;i++)
	{
		for(j=0;j<column-1;j++)
		{
			fprintf(dbFile, "%d, ",result[i*column+j]);
			//__DEBUG__(result[i]);
		}
		fprintf(dbFile, "%d\n",result[i*column+j]);
	}
	fclose(dbFile);
}
/*
("Q0", "select TempR.a from TempR where TempR.b<100000");
("Q1", "select TempR.a from TempR where TempR.b<10000000 and TempR.a<10000000");
("Q2", "select TempR.a from TempR where TempR.b<100000 or TempR.a<100000");
("Q3", "select * from TempR where TempR.b<100000");
("Q4", "select max(TempR.a) from TempR where TempR.b<100000");
("Q5", "select distinct(TempR.a) from TempR where TempR.b<100000");
("Q6", "select * from TempR where TempR.b<1000000 order by TempR.b");
("Q7", "select TempR.b, TempS.b from TempR, TempS where TempR.a=TempS.a");
("Q8", "select TempR.b, TempS.b from TempR, TempS where TempR.a<TempS.a and TempR.b<100000 and TempS.b<1000000");
("Q9", "select TempR.b, TempS.b, TempT.b from TempR, TempS, TempT where TempR.a=TempS.a and TempS.a=TempT.a");
("Q10", "select TempR.b, TempS.b from TempR, TempS where TempR.a=TempS.a order by TempR.b");
("Q11", "select TempR.b, TempS.b from TempR, TempS where TempR.a=TempS.a and TempR.b<TempS.b");
("Q12", "select TempR.b, TempS.b from TempR, TempS where TempR.a=TempS.a and TempR.b<TempS.b");
("Q13", "select TempR.b, TempS.b from TempR, TempS where TempR.a=TempS.a and TempR.b<TempS.b");
*/
void testQuery(int query)
{
	//do some intilization if you need.	
	easedb=new Database();
	easedb->defaultLoad(1);
	const int NUM_QUERY=15;
	char Q[NUM_QUERY][1024]={
		"AGG;TempT;MAX;TempT.a,;$;:JOIN;TempT;TempU;TempT.a,TempU.a,;=,TempT.a,$,$,TempU.a,$,$,;:$:$:$:",
		//"PRO;$;$;TempR.a,;$;:SEL;TempR;$;TempR.b,;<,TempR.b,$,$,100000,$,$,;:$:$:$:",
		"PRO;$;$;TempR.a,;$;:SEL;TempR;$;TempR.a,TempR.b,;AND,<,TempR.b,$,$,10000000,$,$,<,TempR.a,$,$,10000000,$,$,;:$:$:$:15",
		"PRO;$;$;TempR.a,;$;:SEL;TempR;$;TempR.a,TempR.b,;OR,<,TempR.b,$,$,100000,$,$,<,TempR.a,$,$,100000,$,$,;:$:$:$:15",
		"PRO;$;$;TempR.a,TempR.b,;$;:SEL;TempR;$;TempR.b,;<,TempR.b,$,$,100000,$,$,;:$:$:$:",
		"AGG;TempR;MAX;TempR.a,;$;:SEL;TempR;$;TempR.b,;<,TempR.b,$,$,100000,$,$,;:$:$:$:",
		"AGG;TempR;DISTINCT;TempR.a,;$;:SEL;TempR;$;TempR.b,;<,TempR.b,$,$,100000,$,$,;:$:$:$:",
		"PRO;$;$;TempR.a,TempR.b,;$;:ORD;TempR;$;TempR.b,;$;:SEL;TempR;$;TempR.b,;<,TempR.b,$,$,1000000,$,$,;:$:$:$:$:",
		"PRO;$;$;TempR.b,TempS.b,;$;:JOIN;TempR;TempS;TempR.a,TempS.a,;=,TempR.a,$,$,TempS.a,$,$,;:$:$:$:",
		"PRO;$;$;TempR.b,TempS.b,;$;:JOIN;TempR;TempS;TempR.a,TempS.a,;<,TempR.a,$,$,TempS.a,$,$,;:SEL;TempR;$;TempR.b,;<,TempR.b,$,$,100000,$,$,;:$:$:SEL;TempS;$;TempS.b,;<,TempS.b,$,$,1000000,$,$,;:$:$:$:",
		"PRO;$;$;TempR.b,TempS.b,TempT.b,;$;:JOIN;TempR;TempS;TempR.a,TempS.a,;=,TempR.a,$,$,TempS.a,$,$,;:$:JOIN;TempS;TempT;TempS.a,TempT.a,;=,TempS.a,$,$,TempT.a,$,$,;:$:$:$:",
		"PRO;$;$;TempR.b,TempS.b,;$;:ORD;TempR;$;TempR.b,;$;:JOIN;TempR;TempS;TempR.a,TempS.a,;=,TempR.a,$,$,TempS.a,$,$,;:$:$:$:$:",
		"PRO;$;$;TempR.a,TempS.a,;$;:JOIN;TempR;TempS;TempR.a,TempR.b,TempS.a,TempS.b,;AND,=,TempR.a,$,$,TempS.a,$,$,<,TempR.b,$,$,TempS.b,$,$,;:$:$:$:",
		"PRO;$;$;TempR.b,TempS.b,;$;:JOIN;TempR;TempS;TempR.a,TempR.b,TempR.c,TempR.d,TempS.a,TempS.b,TempS.c,TempS.d,;AND,AND,AND,<,TempR.a,$,$,TempS.a,$,$,<,TempR.b,$,$,TempS.b,$,$,<,TempR.c,$,$,TempS.c,$,$,=,TempR.d,$,$,TempS.d,$,$,;:$:$:$:",
		"PRO;$;$;TempR.b,TempS.b,;$;:JOIN;TempR;TempS;TempR.a,TempS.a,;<,TempR.a,$,$,TempS.a,$,$,;:$:$:$:",
		"PRO;$;$;TempT.a,TempU.a,;$;:JOIN;TempT;TempU;TempT.b,TempU.b,;=,TempT.b,$,$,TempU.b,$,$,;:$:$:$:"
	};
	//for(query=0;query<NUM_QUERY;query++)
	query=0;//11;
	{
		__DEBUG__(Q[query]);
		if(strlen(Q[query])>0)
		evaluateQuery(query, Q[query]);
	}
	
	//release some resource if you need.
	
	
}
void testSelection()
{
	easedb=new Database();
	easedb->loadDB("EaseDB.conf");
	const int NUM_QUERY=1;
	char Q[NUM_QUERY][1024]={"AGG;T1;AVG;T1.a3,;$;:JOIN;T1;T2;T1.a1,T2.a1,;=,T1.a1,$,$,T2.a1,$,$,;:SEL;T1;$;T1.a2,;<,T1.a2,$,$,100,$,$,;:$:$:$:$:"	};
	for(int i=0;i<NUM_QUERY;i++)
	{
		cout<<"evaluating "<<Q[i]<<endl;
		evaluateQuery(i, Q[i]);
	}
	
	easedb->dumpDB("EaseDB.conf", false);
}

void testAcess()
{
	int result=0;
	int rLen=1*1024*1024;
	Record *R=(Record*)malloc(sizeof(Record)*rLen);
	Record **Rout=(Record**)malloc(sizeof(Record*));
	CC_CSSTree** tree=(CC_CSSTree**)malloc(sizeof(CC_CSSTree*));
	Database db;
	db.createSortedTable("R1",R,rLen);
	cc_constructCSSTree(R, rLen, tree);
	//(*tree)->print();
	int i;
	for(i=0;i<rLen;i++)
	{
		result=cc_equiTreeSearch(R, rLen, *tree, R[i].value, Rout);
		cout<<i<<", ";
		if(i%32==31) cout<<endl;
		if(result!=1) 
		{
			for(int j=i-result;j<i+result;j++)
				cout<<R[j].value<<", ";
			cout<<"!!!!!"<<result<<endl;
		}
		
	}
	result=cc_equiTreeSearch(R, rLen, *tree, -1, Rout);
	if(result!=1) 
		{
			int j;
			for(j=i-result;j<i+result;j++)
				cout<<R[i].value<<", ";
			cout<<"!!!!!"<<result<<endl;
		}
	db.dropTable("R1");
	
}

void testHash()
{
	int result=0;
	int rLen=1*1024*1024;
	Record *R=(Record*)malloc(sizeof(Record)*rLen);
	Record **Rout=(Record**)malloc(sizeof(Record*));
	CCHashTable **ht=(CCHashTable**)malloc(sizeof(CCHashTable*));
	Database db;
	db.createSortedTable("R1",R,rLen);
	cc_constructHashTable(R,rLen,ht);
	for(int i=0;i<rLen;i++)
	{
		result=cc_hashSearch(R,rLen,*ht,R[i].value,Rout);
		if(result!=1) 
		{
			for(int j=0;j<result;j++)
				cout<<(*Rout)[j].value<<", ";
			cout<<"!!!!!"<<result<<endl;
		}
	}
	db.dropTable("R1");
}

void testSort()
{
	int result=0;
	int rLen=4*1024*1024;
	Record *R=(Record*)malloc(sizeof(Record)*rLen);
	Record **Rout=(Record**)malloc(sizeof(Record*));
	CCHashTable **ht=(CCHashTable**)malloc(sizeof(CCHashTable*));
	Database db;
	db.createSortedTable("R1",R,rLen);
	//cc_mergeSort(R,rLen,Rout);
	cc_radixSort(R,rLen,Rout);
	checkSorted(*Rout,rLen);
	for(int j=0;j<16;j++)
	{
		cout<<(*Rout)[j].value<<", ";
		if(j%8==7)
			cout<<"*"<<endl;
	}
	db.dropTable("R1");
}

void testJoin()
{
	int result=0;
	int rLen=256*1024;
	int sLen=256*1024;
	Record *R=(Record*)malloc(sizeof(Record)*rLen);
	Record *S=new Record[sLen];
	Record **Rout=(Record**)malloc(sizeof(Record*));
	CC_CSSTree** tree=(CC_CSSTree**)malloc(sizeof(CC_CSSTree*));
	Database db;
	db.createSortedTable("R1",R,rLen);
	db.createRandomTable("R2",S,sLen);
	cc_constructCSSTree(R, rLen, tree);
	//result=cc_ninlj(R,rLen,S,sLen,Rout);
	//result=cc_inlj(R,rLen,*tree,S,sLen,Rout);
	//result=cc_smj(R,rLen,S,sLen,Rout);
	//result=cc_hj(R,rLen,S,sLen,Rout);
	cout<<result<<endl;
	for(int j=0;j<result;j++)
	{
		cout<<(*Rout)[j].value<<", ";
		if(j%8==7)
			cout<<"*"<<endl;
	}
	db.dropTable("R1");
	db.dropTable("R2");
}


void SimpleTest(char *query, int mode)
{
	bool execMode=CC_EXEC;
	if(mode==0)
	{
		printf("CC_EXEC, \n");
		execMode=CC_EXEC;
	}
	else
	{
		printf("CO_EXEC, \n");
		execMode=CO_EXEC;
	}
	easedb=new Database();
	easedb->loadDB("RS.conf");
	
	
	
	//char query[256];
	//strcpy(query,"PRO;$;$;a0_4096,;$;:SEL;$;$;a0_4096,;<,a0_4096,$,$,10,$,$,;:$:$:$:");
	//strcpy(query,"JOIN;R;S;R.a0_4096,S.a0_4096,;=,R.a0_4096,$,$,S.a0_4096,$,$,;:$:$:");
	//strcpy(query,"JOIN;R;S;R.a0_1048576,S.a0_1048576,;=,R.a0_1048576,$,$,S.a0_1048576,$,$,;:$:$:");
	//strcpy(query,"JOIN;R;S;R.a0_8392704,S.a0_8392704,;=,R.a0_8392704,$,$,S.a0_8392704,$,$,;:$:$:");
	clock_t l_startTime = clock();
	execQuery(query, mode);
	clock_t l_endTime = clock();
	printf("time, %.3f ms, \n",(l_endTime-l_startTime)*1000/ (double)CLOCKS_PER_SEC);
}


void queryTest(int argc, char ** argv)
{
	int i=0;
	for(i=0;i<argc;i++)
	{
		if(strcmp(argv[i],"-TODS")==0)
		{
			//<queryFileToMicroBenchmark> <numThread> <scale> <CC (0) or CO>
			batchQuery(argv[i+1],atoi(argv[i+2]), atoi(argv[i+3]), atoi(argv[i+4]));
			//batchQuery(argv[i+1],1);
		}
		else if(strcmp(argv[i],"-SINGLE")==0)
		{
			//<query> <numThread> <CC (0) or CO>
			batchQuery2(argv[i+1],atoi(argv[i+2]), atoi(argv[i+3]));
			//batchQuery(argv[i+1],1);
		}
		else if(strcmp(argv[i],"-DUMP")==0)
		{
			easedb=new Database();
			//easedb->dbmbench(atoi(argv[i+1]));
			easedb->dbmbenchForMonet(atoi(argv[i+1]));
			/*char conFileName[100];
			sprintf(conFileName,"EaseDB.conf_%d",atoi(argv[i+1]));
			printf("dump to %s", conFileName);
			easedb->dumpDB(conFileName,true);*/
		}
		else if(i==argc)
		{
			printf("check out the usage\n");
		}
	}
}