#include "CO_Operators.h"
#include "CC_Operators.h"
//selection
int select(Record **Rin, int numColumn, int rLen, PREDICATE_NODE * pRoot, Record **Rout, bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_select(Rin,numColumn,rLen,pRoot, Rout);
	}
	else
	{
		return co_select(Rin,numColumn,rLen,pRoot, Rout);
	}
}

//aggregation
int sum(Record *Rin, int rLen, bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_sum(Rin,rLen);
	}
	else
	{
		return co_sum(Rin,rLen);
	}
}
int avg(Record *Rin, int rLen, bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_avg(Rin,rLen);
	}
	else
	{
		return co_avg(Rin,rLen);
	}
}
int max(Record *Rin, int rLen, bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_max(Rin,rLen);
	}
	else
	{
		return co_max(Rin,rLen);
	}
}

int min(Record *Rin, int rLen, bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_min(Rin,rLen);
	}
	else
	{
		return co_min(Rin,rLen);
	}
}
int count(Record *Rin, int rLen, bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_count(Rin,rLen);
	}
	else
	{
		return co_count(Rin,rLen);
	}
}
int distinct(Record *Rin, int rLen, int col, Record **output, bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_distinct(Rin,rLen, col, output);
	}
	else
	{
		return co_distinct(Rin,rLen, col, output);
	}
}

//sort
void sort(Record *R,int rLen, Record **Rout, int (*fcmp)(const void *, const void *), bool mode)
{
	if(mode==CC_EXEC)
	{
		cc_quickSort(R,rLen, Rout,fcmp);
	}
	else
	{
		if(rLen<2*1024)
			co_quickSort(R,rLen, Rout, fcmp);
		else
			co_partitionSort(R,rLen, Rout);
	}
}


int constructCSSTree(Record *Rin, int rLen, void **tree, bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_constructCSSTree(Rin,  rLen, (CC_CSSTree **)tree);
	}
	else
	{
		return co_constructCSSTree(Rin,  rLen, (VEBTree**)tree);
	}
	
}
int equiTreeSearch(Record *Rin, int rLen, void *tree, int keyForSearch, Record** Rout, bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_equiTreeSearch(Rin,  rLen, (CC_CSSTree *)tree, keyForSearch,Rout);
	}
	else
	{
		return co_equiTreeSearch(Rin,  rLen, (VEBTree *)tree, keyForSearch,Rout);
	}
}
int multi_equiTreeSearch(Record *Rin, int rLen, void *tree, int* searchKeys, int nKey, Record** Rout, bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_multi_equiTreeSearch(Rin,  rLen, (CC_CSSTree *)tree, searchKeys,nKey, Rout);
	}
	else
	{
		return co_multi_equiTreeSearch(Rin,  rLen, (VEBTree *)tree, searchKeys,nKey,Rout);
	}
}

int constructHashTable(Record *Rin, int rLen, void **ht, bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_constructHashTable(Rin,  rLen, (CCHashTable **)ht);
	}
	else
	{
		return co_constructHashTable(Rin,  rLen, (COHashTable**)ht);
	}
	
}
int hashSearch(Record *Rin, int rLen, void *ht , int keyForSearch, Record** Rout, bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_hashSearch(Rin,  rLen, (CCHashTable *)ht, keyForSearch,Rout);
	}
	else
	{
		return co_hashSearch(Rin,  rLen, (COHashTable *)ht, keyForSearch,Rout);
	}
}
int multi_hashSearch(Record *Rin, int rLen, void *ht , int* searchKeys, int nKey, Record** Rout, bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_multi_hashSearch(Rin,  rLen, (CCHashTable *)ht, searchKeys,nKey, Rout);
	}
	else
	{
		return co_multi_hashSearch(Rin,  rLen, (COHashTable *)ht, searchKeys,nKey,Rout);
	}
}

//join
int ninlj(Record *R, int rLen, Record *S, int sLen, PREDICATE_NODE * pRoot, Record** Rout, bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_ninlj(R,rLen,S, sLen,pRoot, Rout);
	}
	else
	{
		return co_ninlj(R,rLen,S, sLen,pRoot, Rout);
	}
}
int multiNLJ(Record **input,int num_col, int rNumColumn, int rLen,int sLen, PREDICATE_NODE * pRoot, Record ** Rout,bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_multiNLJ(input,num_col,rNumColumn,rLen,sLen,pRoot, Rout);
	}
	else
	{
		return co_multiNLJ(input,num_col,rNumColumn,rLen,sLen,pRoot, Rout);
	}

}
int inlj(Record *R, int rLen, void *tree, Record *S, int sLen,PREDICATE_NODE * pRoot, Record** Rout, bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_inlj(R,rLen,(CC_CSSTree*)tree, S, sLen,pRoot, Rout);
	}
	else
	{
		return co_inlj(R,rLen,(VEBTree*)tree, S, sLen,pRoot, Rout);
	}
}
int smj(Record *R, int rLen, Record *S, int sLen, PREDICATE_NODE * pRoot, Record** Rout, bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_smj(R,rLen,S, sLen,pRoot, Rout);
	}
	else
	{
		return co_smj(R,rLen,S, sLen,pRoot, Rout);
	}
}
int hj(Record *R, int rLen, Record *S, int sLen, PREDICATE_NODE * pRoot, Record** Rout, bool mode)
{
	if(mode==CC_EXEC)
	{
		return cc_hj(R,rLen,S, sLen,pRoot, Rout);
	}
	else
	{
		return co_hj(R,rLen,S, sLen,pRoot, Rout);
	}
}

int mergeJoin(Record *R, int rLen, Record *S, int sLen, Record** Rout, bool mode=CC_EXEC)
{
	return co_mergeJoin(R,rLen,S,sLen,Rout);
}
