#include "RowExec.h"
#include "LinkedList.h"
#include "db.h"
#include "qpUtility.h"
#include "time.h"
#include "Operators.h"
#include "config.h"
#include "assert.h"
#include "MyPCL.h"


extern double s_excludeTime;

//aggregation
int dba_sum(DBArray *Rin, int rLen, int attrIndex)
{
	__DEBUG__("dba_sum");
	clock_t l_startTime = clock();
	int i=0;
	int result=0;
	int delta=Rin->numAttr+1;
	for(i=0; i<rLen; i++)
	{
		result+=Rin->data[i*delta+attrIndex];
	}
	clock_t l_endTime = clock();
	printf("time, %.3f ms, \n",(l_endTime-l_startTime)*1000/ (double)CLOCKS_PER_SEC);
	return result;
}
int dba_avg(DBArray *Rin, int rLen, int attrIndex)
{
	__DEBUG__("dba_avg");
	int i=0;
	int result=dba_sum(Rin,rLen,attrIndex);
	if(rLen==0)
		return 0;
	else
		return result/rLen;
}
int dba_max(DBArray *Rin, int rLen, int attrIndex)
{
	__DEBUG__("dba_max");
	int i=0;
	if(rLen==0) return -1;
	int maxValue=Rin->data[attrIndex];
	int delta=Rin->numAttr+1;
	for(i=1; i<rLen; i++)
	{
		if(maxValue<Rin->data[i*delta+attrIndex])
			maxValue=Rin->data[i*delta+attrIndex];
	}
	return maxValue;
}
int dba_min(DBArray *Rin, int rLen, int attrIndex)
{
	__DEBUG__("dba_min");
	int i=0;
	if(rLen==0) return -1;
	int minValue=Rin->data[attrIndex];
	int delta=Rin->numAttr+1;
	for(i=1; i<rLen; i++)
	{
		if(minValue>Rin->data[i*delta+attrIndex])
			minValue=Rin->data[i*delta+attrIndex];
	}
	return minValue;
}

int dba_count(DBArray *Rin, int rLen, int attrIndex)
{
	__DEBUG__("dba_count");
	return rLen;
}
int dba_distinct(DBArray *Rin, int rLen, int attrIndex, DBArray *output, bool mode)
{
	__DEBUG__("dba_distinct");
	int result=0;
	Record **tempR=(Record**)malloc(sizeof(Record*));
	Record **tempOutput=(Record**)malloc(sizeof(Record*));
	result=Rin->extractToRecord(tempR,attrIndex);
	result=distinct(*tempR, result, 1, tempOutput, mode);
	DBArray::transform(Rin, *tempOutput,result,output);
	free(tempOutput);
	free(tempR);
	return result;
}


int dba_sort(DBArray *Rin, int attrIndex, DBArray *output, bool mode)
{
	__DEBUG__("dba_sort");
	int result=0;
	Record **tempR=(Record**)malloc(sizeof(Record*));
	Record **tempOutput=(Record**)malloc(sizeof(Record*));
	result=Rin->extractToRecord(tempR,attrIndex);
	sort(*tempR, result, tempOutput, compareRecord,mode);
	DBArray::transform(Rin, *tempOutput,result,output);
	free(tempOutput);
	free(tempR);
	return result;
}

int dba_cc_ninlj(DBArray *R, int rLen, DBArray *S, int sLen, Record** Rout)
{
	__DEBUG__("dba_cc_ninlj");
	int numResult = 0;
	int i=0;
	int j=0;
	int k=0;
	int m=0;
	int startIndex=0;
	int endIndex=0;
	//LinkedList *ll=(LinkedList*)malloc(sizeof(LinkedList)); ll->init();
	LinkedList *ll=(LinkedList*)malloc(sizeof(LinkedList));
	ll->init();
	int minLen=(R->numAttr>S->numAttr)?S->numAttr:R->numAttr;
	int nlj_block_size=L2_CAP/((minLen+1)*sizeof(int)*4);
	int numBlock=rLen/nlj_block_size;
	if (rLen%nlj_block_size!=0) 
		numBlock=numBlock+1;
	Record r;	
	int dataS[2];
	int deltaR=R->numAttr+1;
	int deltaS=S->numAttr+1;
	
	int curK=0;
	__DEBUG__("sample predicate, two attributes");
	for(i=0;i<(numBlock-1);i++)
	{
		startIndex=i*nlj_block_size;
		endIndex=(i+1)*nlj_block_size;
		for(j=0;j<sLen;j++)
		{
			for(curK=0;curK<2;curK++)
				dataS[curK]=S->data[j*deltaS+curK];
			for(k=startIndex;k<endIndex;k++)
			{
				if(dataS[0]==R->data[k*deltaR] && dataS[1]==R->data[k*deltaR+1])
				{	
					r.rid=R->data[k*deltaR];
					r.value=S->data[j*deltaS];
					ll->fill(&r);
					numResult++;				
				}
			}
		}
	}
	//for the last block.
	startIndex=i*nlj_block_size;
	endIndex=rLen;
	for(j=0;j<sLen;j++)
	{
		for(curK=0;curK<2;curK++)
				dataS[curK]=S->data[j*deltaS+curK];
		for(k=startIndex;k<endIndex;k++)
		{
			
			if(dataS[0]==R->data[k*deltaR] && dataS[1]==R->data[k*deltaR+1])
			{
				r.rid=R->data[k*deltaR];
				r.value=S->data[j*deltaS];
				ll->fill(&r);
				numResult++;
			}
		}
	}
	if(numResult!=0)
	{
		*Rout=(Record*)malloc(sizeof(Record)*numResult);
		ll->copyToArray(*Rout);
	}
	delete ll;
	return numResult;
}


int dba_CONLJ(DBArray *R, DBArray *S, int rStart, int rEnd, int sStart, int sEnd, LinkedList *ll)
{
	int result=0;
	int curK=0;
	if(rStart+CUTOFF_ROW>=rEnd)
	{
		Record r;
		int deltaS=S->numAttr+1;
		int deltaR=R->numAttr+1;
		int dataS[2];
		for(int i=sStart;i<sEnd;i++)
		{
			for(curK=0;curK<2;curK++)
				dataS[curK]=S->data[i*deltaS+curK];
			for(int j=rStart;j<rEnd;j++)
			{
				if(dataS[0]==R->data[j*deltaR] && dataS[1]==R->data[j*deltaR+1])
				{	
					r.rid=R->data[j*deltaR];
					r.value=S->data[i*deltaS];
					ll->fill(&r);
					result++;	
				}
			}
		}
	}
	else
		if(sStart+CUTOFF_ROW>=sEnd)
		{
			Record r;
			int deltaS=S->numAttr+1;
			int deltaR=R->numAttr+1;
			int dataR[2];
			for(int i=rStart;i<rEnd;i++)
			{
				for(curK=0;curK<2;curK++)
					dataR[curK]=R->data[i*deltaR+curK];
				for(int j=sStart;j<sEnd;j++)
				{
					if(dataR[0]==S->data[j*deltaS] && dataR[1]==S->data[j*deltaS+1])
					{	
						r.rid=R->data[i*deltaR];
						r.value=S->data[j*deltaS];
						ll->fill(&r);
						result++;	
					}
				}
			}
		}
		else
		{
			int midS=(sStart+sEnd)/2;
			int midR=(rStart+rEnd)/2;
			//interesting order
			result+=dba_CONLJ(R,S,rStart,midR,sStart,midS,ll);
			result+=dba_CONLJ(R,S,midR,rEnd,sStart,midS,ll);
			result+=dba_CONLJ(R,S,midR,rEnd,midS,sEnd,ll);
			result+=dba_CONLJ(R,S,rStart,midR,midS,sEnd,ll);
		}
	return result;
}

int dba_co_ninlj(DBArray *R, int rLen, DBArray *S, int sLen, Record** Rout)
{
	__DEBUG__("dba_co_ninlj");
	int numResult = 0;
	int i=0;
	int j=0;
	int k=0;
	int m=0;
	int startIndex=0;
	int endIndex=0;
	__DEBUG__("simplePredicate two attributes");
	LinkedList *ll=(LinkedList*)malloc(sizeof(LinkedList)); ll->init();
	int minLen=(R->numAttr>S->numAttr)?S->numAttr:R->numAttr;
	assert(minLen>=2);
	numResult=dba_CONLJ(R,S,0,rLen,0,sLen,ll);
	if(numResult!=0)
	{
		*Rout=(Record*)malloc(sizeof(Record)*numResult);
		ll->copyToArray(*Rout);
	}
	delete ll;
	return numResult;
}

int dba_ninlj(DBArray *R, DBArray *S, bool mode)
{
	__DEBUG__("dba_hj");
	int result=0;
	Record **Rout=(Record**)malloc(sizeof(Record*));
	if(mode==CC_EXEC)
	{
		result=dba_cc_ninlj(R,R->numTuple,S,S->numTuple,Rout);
	}
	else
	{
		result=dba_co_ninlj(R,R->numTuple,S,S->numTuple,Rout);
	}
	if(result!=0)
	free(*Rout);
	free(Rout);
	return result;
}
int dba_inlj(DBArray *R,int attrIndex1, DBArray *S, int attrIndex2,bool mode)
{
	__DEBUG__("dba_inlj");
	int result=0;
	s_excludeTime=0;
	MyPCL *pcl=new MyPCL();
	pcl->OpenPCL();
	Record **tempR=(Record**)malloc(sizeof(Record*));
	int len1=R->extractToRecord(tempR,attrIndex1);
	qsort(*tempR,len1,sizeof(Record),compareRecord);
	void **tree=(void**)malloc(sizeof(void*));
	constructCSSTree(*tempR,len1,tree,mode);
	Record **tempS=(Record**)malloc(sizeof(Record*));
	int len2=S->extractToRecord(tempS,attrIndex2);
	s_excludeTime=pcl->ClosePCL();
	pcl->ClearCache();

	Record **tempOutput=(Record**)malloc(sizeof(Record*));
	result=inlj(*tempR,len1,*tree,*tempS,len2,NULL,tempOutput,mode);
	free(*tree);
	free(tree);
	free(*tempOutput);
	free(*tempR);
	free(*tempS);
	free(tempR);
	free(tempS);
	return result;
}
int dba_smj(DBArray *R,int attrIndex1, DBArray *S, int attrIndex2,bool mode)
{
	__DEBUG__("dba_smj");
	int result=0;
	Record **tempR=(Record**)malloc(sizeof(Record*));
	int len1=R->extractToRecord(tempR,attrIndex1);
	Record **tempS=(Record**)malloc(sizeof(Record*));
	int len2=S->extractToRecord(tempS,attrIndex2);

	Record **tempOutput=(Record**)malloc(sizeof(Record*));
	result=smj(*tempR,len1,*tempS,len2,NULL,tempOutput,mode);
	free(*tempOutput);
	free(*tempR);
	free(*tempS);
	free(tempOutput);
	free(tempR);
	free(tempS);
	return result;	
}
int dba_hj(DBArray *R, int attrIndex1, DBArray *S, int attrIndex2, bool mode)
{
	__DEBUG__("dba_hj");
	int result=0;
	Record **tempR=(Record**)malloc(sizeof(Record*));
	int len1=R->extractToRecord(tempR,attrIndex1);
	Record **tempS=(Record**)malloc(sizeof(Record*));
	int len2=S->extractToRecord(tempS,attrIndex2);

	Record **tempOutput=(Record**)malloc(sizeof(Record*));
	result=hj(*tempR,len1,*tempS,len2,NULL,tempOutput,mode);
	free(*tempOutput);
	free(*tempR);
	free(*tempS);
	free(tempOutput);
	free(tempR);
	free(tempS);
	return result;
}



