#include <iostream>
#include "Database.h"
#include "UInterface.h"
#include "QueryPlanTree.h"
#include "evaluateFuction.h"
#include "QueryPlanNode.h"
#include "MyPCL.h"

//CC
#include "CC_Operators.h"
#include "CC_CSSTree.h"
//CO
#include "CC_Operators.h"
#include "CC_CSSTree.h"

#include "batchQuery.h"

#include "testPrimitive.h"
#include "DBArray.h"
#include "RowExec.h"
#include "UnitTest.h"
#include <vector>
using namespace std;
const int numTestOpInRow=10;
char* opStr[numTestOpInRow]={
	"U_MIN", "U_MAX", "U_SUM", "U_COUNT",
	"U_DISTINCT", "U_SORT", "U_NINLJ", "U_INLJ",
	"U_SMJ", "U_HJ"
};

extern double s_excludeTime;

void test_internal(int op, DBArray *dba, DBArray *dbb, bool mode)
{
	switch(op)
	{
	case U_MIN:
		cout<<dba_min(dba,dba->numTuple, mode)<<endl;
		break;
	case U_MAX:
		cout<<dba_max(dba,dba->numTuple, mode)<<endl;
		break;
	case U_SUM:
		cout<<dba_sum(dba,dba->numTuple, mode)<<endl;
		break;
	case U_COUNT:
		cout<<dba_count(dba,dba->numTuple, mode)<<endl;
		break;
	case U_DISTINCT:
		{
			DBArray *output=new DBArray();
			cout<<dba_distinct(dba,dba->numTuple, 1,output, mode)<<endl;
		}
		break;
	case U_SORT:
		{
			DBArray *output=new DBArray();
			cout<<dba_sort(dba,1,output,mode)<<endl;
		}
		break;
	case U_NINLJ:
		cout<<dba_ninlj(dba,dbb,mode)<<endl;
		break;
	case U_INLJ:
		cout<<dba_inlj(dba,1,dbb,1,mode)<<endl;
		break;
	case U_SMJ:
		cout<<dba_smj(dba,1,dbb,1,mode)<<endl;
		break;
	case U_HJ:
		cout<<dba_hj(dba,1,dbb,1,mode)<<endl;
		break;
	default:
		cout<<"nothing done in the row_test_internal:"<<op<<endl;
		break;
	}
}

void row_test(int op)
{
	DBArray *dba=new DBArray();
	dba->importDB("S.a0_rowdemo.dat");
	dba->check();
	DBArray *dbb=new DBArray();
	dbb->importDB("R.a0_rowdemo.dat");
	dbb->check();
	vector<double> cc_elapsedTime;
	vector<double> co_elapsedTime;
	double t=0;
	MyPCL *pcl=new MyPCL();
	if(U_ALL==op)
	{
		int i=0;
		for(i=0;i<numTestOpInRow;i++)
		{
			pcl->ClearCache();
			pcl->OpenPCL();
			test_internal(i,dba,dbb,CC_EXEC);
			t=pcl->ClosePCL();
			t=t-s_excludeTime;
			cc_elapsedTime.push_back(t);
		}
		for(i=0;i<numTestOpInRow;i++)
		{
			pcl->ClearCache();
			pcl->OpenPCL();
			test_internal(i,dba,dbb,CO_EXEC);
			t=pcl->ClosePCL();
			t=t-s_excludeTime;
			co_elapsedTime.push_back(t);
		}
		cout<<"\n------------CC engine----------------"<<endl;
		for(i=0;i<numTestOpInRow;i++)
		{
			cout<<opStr[i]<<", time, "<<cc_elapsedTime[i]<<endl;
		}
		cout<<endl;
		cout<<"\n------------CO engine----------------"<<endl;
		for(i=0;i<numTestOpInRow;i++)
		{
			cout<<opStr[i]<<", time, "<<co_elapsedTime[i]<<endl;
		}
		cout<<endl;
	}
	else
	{
		//cc
		pcl->ClearCache();
		pcl->OpenPCL();
		test_internal(op,dba,dbb,CC_EXEC);
		t=pcl->ClosePCL();
		t=t-s_excludeTime;
		cc_elapsedTime.push_back(t);
		//co
		pcl->ClearCache();
		pcl->OpenPCL();
		test_internal(op,dba,dbb,CO_EXEC);
		t=pcl->ClosePCL();
		t=t-s_excludeTime;
		co_elapsedTime.push_back(t);
		cout<<"\n------------CC engine----------------"<<endl;
		cout<<opStr[op]<<", time, "<<cc_elapsedTime[0]<<endl;
		cout<<endl;
		cout<<"\n------------CO engine----------------"<<endl;
		cout<<opStr[op]<<", time, "<<co_elapsedTime[0]<<endl;
		cout<<endl;

	}
}


void column_test(int op)
{
	DBArray *dba=new DBArray();
	dba->importDB("R.a0_columndemo.dat");
	dba->check();
	DBArray *dbb=new DBArray();
	dbb->importDB("S.a0_columndemo.dat");
	dbb->check();
	vector<double> cc_elapsedTime;
	vector<double> co_elapsedTime;
	double t=0;
	MyPCL *pcl=new MyPCL();
	if(U_ALL==op)
	{		
		int i=0;
		for(i=0;i<numTestOpInRow;i++)
		{
			pcl->ClearCache();
			pcl->OpenPCL();
			test_internal(i,dba,dbb,CC_EXEC);
			t=pcl->ClosePCL();
			t=t-s_excludeTime;
			cc_elapsedTime.push_back(t);
		}
		for(i=0;i<numTestOpInRow;i++)
		{
			pcl->ClearCache();
			pcl->OpenPCL();
			test_internal(i,dba,dbb,CO_EXEC);
			t=pcl->ClosePCL();
			t=t-s_excludeTime;
			co_elapsedTime.push_back(t);
		}
		cout<<"\n------------CC engine----------------"<<endl;
		for(i=0;i<numTestOpInRow;i++)
		{
			cout<<opStr[i]<<", time, "<<cc_elapsedTime[i]<<endl;
		}
		cout<<endl;
		cout<<"\n------------CO engine----------------"<<endl;
		for(i=0;i<numTestOpInRow;i++)
		{
			cout<<opStr[i]<<", time, "<<co_elapsedTime[i]<<endl;
		}
		cout<<endl;
	}
	else
	{
		//cc
		pcl->ClearCache();
		pcl->OpenPCL();
		test_internal(op,dba,dbb,CC_EXEC);
		t=pcl->ClosePCL();
		t=t-s_excludeTime;
		cc_elapsedTime.push_back(t);
		//co
		pcl->ClearCache();
		pcl->OpenPCL();
		test_internal(op,dba,dbb,CO_EXEC);
		t=pcl->ClosePCL();
		t=t-s_excludeTime;
		co_elapsedTime.push_back(t);
		cout<<"\n------------CC engine----------------"<<endl;
		cout<<opStr[op]<<", time, "<<cc_elapsedTime[0]<<endl;
		cout<<endl;
		cout<<"\n------------CO engine----------------"<<endl;
		cout<<opStr[op]<<", time, "<<co_elapsedTime[0]<<endl;
		cout<<endl;
	}

	
	
}


void query_test(int op, int numThreads, bool mode)
{
	const int numTestQueries=2;
	char *sampleQuery[numTestQueries]={
		"AGG;R;SUM;R.a00,;$;:ORD;R;$;R.a00,;$;:$:$:$:",
		"JOIN;R;S;R.a00,S.a00,;=,R.a00,$,$,S.a00,$,$,;:$:$:"
	};
	if(op==U_SORT)
	{
		initDB2("RS.conf");
		batchQuery2(sampleQuery[0],numThreads,mode);
	}
	else
		if(op==U_HJ)
		{
			initDB2("RS.conf");
			batchQuery2(sampleQuery[0],numThreads,mode);
		}
		else
		{
			cout<<"please choose U_SORT or U_HJ"<<endl;
		}
}


void exec_query(char *query, bool mode)
{
	initDB2("RS.conf");
	MyPCL *pcl=new MyPCL();
	pcl->ClearCache();
	pcl->OpenPCL();
	execQuery(query, mode);
	double t=pcl->ClosePCL();
	cout<<endl;
	if(mode==CC_EXEC)
	{
		cout<<"Query, "<<query<<", engine, CC, time, "<<t<<endl;
	}
	else
	{
		cout<<"Query, "<<query<<", engine, CO, time, "<<t<<endl;
	}
}