#include "batchQuery.h"
#include "time.h"
#include "MyPCL.h"
#include "Database.h"
#include "UInterface.h"
#include "QueryPlanTree.h"
#include "evaluateFuction.h"
#include "QueryPlanNode.h"

//CC
#include "CC_Operators.h"
#include "CC_CSSTree.h"
//CO
#include "CC_Operators.h"
#include "CC_CSSTree.h"

#include <windows.h>
#include <strsafe.h>
#include <fstream>
#include <string>
#include <vector>
using namespace std;

extern Database *easedb;

struct ws_query{
	char *queryFileName;
	char *query;
	bool execMode;
};

#define QUERY_LENGTH 512



void execQuery(char *query, bool execMode)
{
	QueryPlanTree tree(execMode);
	tree.buildTree(query);
	clock_t l_startTime = clock();
	tree.execute();
	clock_t l_endTime = clock();
	int len=tree.planStatus->numResultColumn*tree.planStatus->numResultRow;
	free(tree.planStatus->finalResult);
	printf("ExecuteQuery, %s, %d",query,len);
	printf("time, %.3f ms, \n",(l_endTime-l_startTime)*1000/ (double)CLOCKS_PER_SEC);
}

DWORD WINAPI ThreadProc_query( LPVOID lpParam ) 
{ 
    ws_query* pData;
    pData = (ws_query*)lpParam;
	char* queryFileName=pData->queryFileName;
	char query[QUERY_LENGTH];
	
	//do querying
	ifstream fin;
	fin.open(queryFileName);
	vector<string> queryVec;
	int numQuery=0;
	while(!fin.eof())
	{
		fin.getline(query,QUERY_LENGTH);
		if(strlen(query)<1) break;
		string str;
		str.assign(query);
		queryVec.push_back(str);
		numQuery++;
	}
	int i=0;
	MyPCL *localPcl=(MyPCL*)malloc(sizeof(MyPCL));
	for(i=0;i<numQuery;i++)
	{
		localPcl->OpenPCL();
		execQuery((char*)queryVec[i].data(), pData->execMode);
		localPcl->ClosePCL();
		
	}
    return 0; 
} 

//just do a query.
DWORD WINAPI ThreadProc_query2( LPVOID lpParam ) 
{ 
    ws_query* pData;
    pData = (ws_query*)lpParam;
	char* query=pData->query;
	MyPCL *localPcl=(MyPCL*)malloc(sizeof(MyPCL));
	localPcl->OpenPCL();
	execQuery(query, pData->execMode);
	localPcl->ClosePCL();
	return 0; 
} 




void batchQuery(char *queryFileName, int numThread, int scale, int mode)
{
	bool execMode=CC_EXEC;
	if(mode ==0)
		execMode=CC_EXEC;
	else
		execMode=CO_EXEC;
	easedb=new Database();
	//easedb->loadDB("EaseDB.conf");
	easedb->dbmbenchForMonetLoad(scale);
	ws_query* pData;
	DWORD* dwThreadId=new DWORD[numThread];
    HANDLE* hThread=new HANDLE[numThread]; 
	char tFileName[128];
    int i=0;
	MyPCL *pcl=(MyPCL*)malloc(sizeof(MyPCL));
	pcl->OpenPCL();
    for( i=0; i<numThread; i++ )
    {
        // Allocate memory for thread data.
        pData = (ws_query*) HeapAlloc(GetProcessHeap(),
                HEAP_ZERO_MEMORY, sizeof(ws_query));

        if( pData == NULL )
            ExitProcess(2);

        // Generate unique data for each thread.
		sprintf(tFileName,"%s_%d",queryFileName,i);
		pData->queryFileName=tFileName;
		pData->execMode=execMode;
		
		
        hThread[i] = CreateThread( 
            NULL,              // default security attributes
            0,                 // use default stack size  
            ThreadProc_query,        // thread function 
            pData,             // argument to thread function 
            0,                 // use default creation flags 
            &dwThreadId[i]);   // returns the thread identifier 
		//assign to different processors
		SetThreadAffinityMask(hThread[i],1<<i);
        // Check the return value for success.		
        if (hThread[i] == NULL) 
        {
            ExitProcess(i);
        }
    }
    // Wait until all threads have terminated.
    WaitForMultipleObjects(numThread, hThread, TRUE, INFINITE);
	
	// Close all thread handles upon completion.
	pcl->ClosePCL(numThread);

    for(i=0; i<numThread; i++)
    {
        CloseHandle(hThread[i]);
    }
	delete [] dwThreadId;
	delete [] hThread;
}


void initDB2(char *confFile)
{
	easedb=new Database();
	easedb->loadDB(confFile);
}


void batchQuery2(char *query, int numThread,int mode)
{
	bool execMode=CC_EXEC;
	if(mode ==0)
	{
		execMode=CC_EXEC;
		cout<<"query, "<<query<<", numThread, "<<numThread<<", CC_EXEC"<<endl;
	}
	else
	{
		execMode=CO_EXEC;
		cout<<"query, "<<query<<", numThread, "<<numThread<<", CO_EXEC"<<endl;
	}
	ws_query* pData;
	DWORD* dwThreadId=new DWORD[numThread];
    HANDLE* hThread=new HANDLE[numThread]; 
    int i=0;
	MyPCL *pcl=(MyPCL*)malloc(sizeof(MyPCL));
	pcl->OpenPCL();
    for( i=0; i<numThread; i++ )
    {
        // Allocate memory for thread data.
        pData = (ws_query*) HeapAlloc(GetProcessHeap(),
                HEAP_ZERO_MEMORY, sizeof(ws_query));

        if( pData == NULL )
            ExitProcess(2);

        // Generate unique data for each thread.
		pData->query=new char[256];
		strcpy(pData->query,query);
		pData->execMode=execMode;
		
		
        hThread[i] = CreateThread( 
            NULL,              // default security attributes
            0,                 // use default stack size  
            ThreadProc_query2,        // thread function 
            pData,             // argument to thread function 
            0,                 // use default creation flags 
            &dwThreadId[i]);   // returns the thread identifier 
		//assign to different processors
		SetThreadAffinityMask(hThread[i],1<<i);
        // Check the return value for success.		
        if (hThread[i] == NULL) 
        {
            ExitProcess(i);
        }
    }
    // Wait until all threads have terminated.
    WaitForMultipleObjects(numThread, hThread, TRUE, INFINITE);
	
	// Close all thread handles upon completion.
	cout<<"\n---all thread completes\n"<<endl;
	pcl->ClosePCL(numThread);

    for(i=0; i<numThread; i++)
    {
        CloseHandle(hThread[i]);
    }
	delete [] dwThreadId;
	delete [] hThread;
}